<?php
##### rrtextplus_ancestor.php, installed by ###TextPlus Charts, by Robin Richmond
/* The TextPlus Ancestry chart is a distinctive style of Ancestry chart that uses
horizontal space very efficiently. It allows 16 generations or so to br printed
in the width of one printed page.

It displays alternate birth and death date and place when it there is no birth date, birth place,
death date, or death place.
It also goes to a lot of trouble to try to wrap text to a print margin.

(I have also implemented a separate mod (not yet published) that abbreviates place names,
and thus reduces the need to wrap the data for a person to multiple lines.
See the Wiki article on TextPlus Charts, and the TextPlus help file for more information.)

THIS PROGRAM CAN ONLY HANDLE ONE SET OF PARENTS FOR A PERSON
########################### History ##############################
************************************************************/
# These parameters help allow the two Text+ Program to use identical parameterized code.
$params['charttype'] = 'ancestor';
$params['phpBaseFilename'] = 'rrtextplus_' . $params['charttype'];
$params['phpFilename'] = $params['phpBaseFilename'] . '.php';
$tngModNames['TextPlus Charts'][$params['phpFilename']] = '17beta1'; #Register this mod
########################################################

@set_time_limit(0);
$textpart = "pedigree"; #Gets the "pedigree" section of the language $text variables.
include("tng_begin.php");
include($subroot . "pedconfig.php");
################################################
#Read custom language strings from a mod subfolder rather than from regular cust_text.php files.
$modfolder = $text['tp-modfolder'];
if (file_exists("$modspath/$modfolder/{$mylanguage}_cust_text.php"))
	include("$modspath/$modfolder/{$mylanguage}_cust_text.php");
else include("$modspath/$modfolder/languages/English_cust_text.php");

#PHP functions installed by the Text Plus Charts mod common to both Text+ Chart Programs
include("{$cms['tngpath']}rrtextplus_lib.php");

###Call Graceful Exit if the mod has been installed.
if (file_exists('rrgracefulexit.php')) {
	include_once('rrgracefulexit.php');
	rrGracefulExit();
}
#Check expected querystring parameters for values.
if (!isset($siblings)) $siblings = 0;
if (!isset($showgennums)) $showgennums = 0;
if (!isset($siblingnums)) $siblingnums = 0;
if (!isset($compact)) $compact = 0;
if (!isset($generations)) $generations = '';
if (!isset($_GET['gfocus']) || $_GET['gfocus']=='') $gfocus = 1;

#Initialize mod options and formatting parameters.
initializeParameters($params['charttype']);
$params['siblings'] = $siblings;
$params['showgennums'] = $showgennums;
$params['siblingnums'] = $siblings && $siblingnums;
$params['compact'] = $compact;

# Get the URLs of programs that link to the other charts in the inner menu.
$getperson_url = getURL( "getperson", 1 );
$ahnentafel_url = getURL( "ahnentafel", 1 );
$pedigree_url = getURL( "pedigree", 1 );
$pedigreetext_url = getURL( "pedigreetext", 1 );
$textplus_url = getURL( $params['phpBaseFilename'], 1 );
$extrastree_url = getURL( "extrastree", 1 );
$ultraped_url = getURL( "ultraped", 1 );
$pdfform_url = getURL( "rpt_pdfform", 1 );
$vertical_url = getURL( "verticalchart", 1 );
$fan_url = getURL( "fan", 1 );

$startingpersonID = $personID;
$result = getPersonFullPlusDates($tree, $startingpersonID);
if( $result ) {
	$row = tng_fetch_assoc( $result );
	$righttree = checktree($tree);
	$rightbranch = $righttree ? checkbranch($row['branch']) : false;
	$rights = determineLivingPrivateRights($row, $righttree, $rightbranch);
	$row['allow_living'] = $rights['living'];
	$row['allow_private'] = $rights['private'];
	$namestr = getName( $row );
	$logname = $tngconfig['nnpriv'] && $row['private'] ? $admtext['text_private'] : ($nonames && $row['living'] ? $text['living'] : $namestr);
}

$treeResult = getTreeSimple($tree);
$treerow = tng_fetch_assoc($treeResult);
$disallowgedcreate = $treerow['disallowgedcreate'];
$allowpdf = !$treerow['disallowpdf'] || ($allow_pdf && $rightbranch);
tng_free_result($treeResult);

$bookmarkstring = "<a href='{$textplus_url}personID=$personID&amp;tree=$tree&amp;generations=$generations'>Text+ {$text['tp-'.$params['charttype']]} $namestr ($personID)</a> $generations {$text['generations']}";
writelog($bookmarkstring);
preparebookmark($bookmarkstring);

$flags['tabs'] = $tngconfig['tabs'];
$flags['scripting'] = "<script type='text/javascript'>var tnglitbox;</script>\n";

tng_header( "{$text['tp-'.$params['charttype']]} $namestr", $flags );

#TextPlus CSS
echo "<link href='css/rrtextplus.css' rel='stylesheet' type='text/css'>\n";

$photostr = showSmallPhoto( $personID, $namestr, $rights['both'], 0, false, $row['sex'] );
echo tng_DrawHeading( $photostr, "{$text['tp-'.$params['charttype']]} <span class='nw'>$namestr</span>", getYears( $row ) );

#We may prefix marriage data with "m" or "parents", depending on whether we display the data with
#the child or with both parents.
if ($rrconfig['Text+AncestorMarriage'] == 'parents')
	$params['marriageprefix'] = $text['tp-married1'];
else $params['marriageprefix'] = tng_strtolower($text['parents']);

# $vbars - a 30 element control array (accommodating 30 generations), where each value can be 0, 1, 2, or 3
#          to indicate which character in $drawvbar should be printed for that generation.
$vbars = array_fill(0,30,0);

# $counts - A global array of counts used throughout the program
$count = array('generations'=>0, 'Ancestors'=>0, 'Siblings'=>0, 'leaflinks'=>0,
	'BlankLines'=>0, 'Lines'=>0);
$count['CarriedIn'] = array();
$count['MaleLines'] = array();
$count['FemaleLines'] = array();
### End of initialization

#The number of Generations processed by one chart in the Inner Manu is normally passed to
#the other charts on the same Inner Menu (i.e. Ancestors vs Descendants). But the Text+ Charts
#can handle more generations than most of the other charts, so the Text+ Chart programs
#must limit the Generations parameter they pass to the other chart programs.
#For the Text+ Ancestor Chart, generation limits are imposed this way:
# - The three variants of $pedigree.php (Standard, compact, and box) will be given the value
#   of the TNG system variable for "Initial Generations"; $pegigree['initpedgens']
# - The remaining charts (vertical, text, ahnentafel, Fan, Media, etc) will be given the
#   number of generations specified by this program - but limited by the TNG system variable
#   for the maximum number of generations ($pedigree['maxgen']).
$tnginitgen = $pedigree['initpedgens'] ? $pedigree['initpedgens'] : 4;
$tngmaxgen = $pedigree['maxgen'] ? $pedigree['maxgen'] : 16;
if (!isset($generations)) $generations=$tnginitgen;
elseif (!$generations) $generations=$tngmaxgen;
if ($generations<$tnginitgen) $linkgens = $generations;
else $linkgens = $tnginitgen;

if (!isset($parentset)) $parentset=0;  #query-string parameter used by ancestor charts
######### Inner Menu line 1
	$innermenu = "<input type='hidden' name='personID' value='$personID'>";
	$innermenu .= "<input type='hidden' name='tree' value='$tree'>";
	$innermenu .= "<div>";
	$innermenu .= generateChartTypeLabel($params['charttype']);

	$innermenu .= "<a href='$pedigree_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;display=standard&amp;generations=$tnginitgen' class='lightlink' id='stdpedlnk'>{$text['pedstandard']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$vertical_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;display=vertical&amp;generations=$linkgens\" class=\"lightlink\" id=\"pedchartlnk\">{$text['pedvertical']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$pedigree_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;display=compact&amp;generations=$tnginitgen\" class=\"lightlink\" id=\"compedlnk\">{$text['pedcompact']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$pedigree_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;display=box&amp;generations=$tnginitgen\" class=\"lightlink\" id=\"boxpedlnk\">{$text['pedbox']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$pedigreetext_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$linkgens\" class=\"lightlink\">{$text['pedtextonly']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	#The new TextPlus link always follows the Text version of the chart
	$innermenu .= "<a href='{$textplus_url}personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$linkgens' class='lightlink3' id='textlnk\'>{$text['tp-textplus']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$ahnentafel_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$linkgens\" class=\"lightlink\">{$text['ahnentafel']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$fan_url = getURL('fanchart', 1);
	$innermenu .= "<a href=\"$fan_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$generations\" class=\"lightlink\" id=\"fanlnk\">{$text['fanchart']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$extrastree_url" . "personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;showall=1&amp;generations=$linkgens\" class=\"lightlink\">{$text['media']}</a>\n";

	#If links.php has been installed by the mod "Male and Female Descendant and Parent , link to it.
	if (file_exists('lines.php')) {
		if (empty($text['lines'])) $text['lines'] = 'Parent&nbsp;Lines';
		$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href='lines.php?personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;showall=1&amp;generations=$linkgens' class='lightlink'>{$text['lines']}</a>\n";
	}
	#Include the ancestors map only if the ancestors map program exists.
	if (file_exists('mappedigree.php')) {
		if (empty($text['ancestorsmap'])) $text['ancestorsmap'] = 'map';
		$innermenu .= "&nbsp;&nbsp; | &nbsp;&nbsp; <a href=\"mappedigree.php?personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$linkgens\" class=\"lightlink\" id=\"maplnk\">{$text['ancestorsmap']}</a>\n";
	}
	if($allowpdf)
		$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href=\"#\" class=\"lightlink\" onclick=\"tnglitbox = new LITBox('$pdfform_url" . "pdftype=ped&amp;personID=$personID&amp;tree=$tree&amp;generations=$tnginitgen',{width:350,height:350});return false;\">PDF</a>\n";
	$innermenu .= "</div>\n"; #end of line 1 of the innermenu

######### Second Inner Menu line, with Chart Content options
	#These controls are placed in the main window Inner Menu and in the Format for Printing window
	#Option 1 - Generations selection box, which is built by a function because it is used
	#in both Text+ Chart programs.
	$commonmenu = generateGenerationsSelectionBox($params['gensallowed'], $params['genstodo']);
	# Option 2 - Display Generation #
	#When this checkbox is not checked, the show sibling #'s checkbox is disabled and a warning
	#message is saved to the the label element that wraps the show sibling # checkbox.
	$checked = $showgennums ? ' checked' : '';
	$commonmenu .= "&nbsp; <label class='nw'>
		<input type='checkbox' class='tpmenu' name='showgennums' id='showgennums' value='yes'$checked
			onclick = 'return submitOrSetSiblingNumbers()'/>{$text['tp-gennums']}</label>\n";

	# Option 3- Display Siblings
	$checked = $siblings ? ' checked' : '';
	$commonmenu .= "&nbsp; <label class='nw'><input type='checkbox' class='tpmenu' name='siblings' id='siblings'
		value='yes'$checked onclick = 'return submitOrSetSiblingNumbers()'/>{$text['tp-frmsiblings']}</label>\n";

	### Option 4 - Display Sibling count - Is enabled only if Generation #s is checked.
	#If it is disabled, a pop-up message is generated in the title attribute of the enclosing <label> tag.
	$checked = $siblingnums ? ' checked' : '';
	if ($siblings && $showgennums) {
		$siblingnumsmessage = $disabled = $disabledclass = '';
	} else {
		$disabled = ' disabled';
		$siblingnumsmessage = $text['tp-sibsgennumrequired'];
		$disabledclass = ' disabledlabel';
	}
	$commonmenu .="&nbsp; <label class='nw$disabledclass' id='siblingslabel' title=\"$siblingnumsmessage\">
		<input type='checkbox' name='siblingnums' id='siblingnums' class='tpmenu' value='yes'$checked$disabled
			onclick = 'return selfSubmit();'>{$text['tp-siblingnums']}</label>\n";

	# Option 5 - Compact layout
	$checked = $compact ? ' checked' : '';
	$commonmenu .="&nbsp; <label class='nw' title=\"{$text['tp-compactmsg']}\">
		<input type='checkbox' name='compact' id='compact' class='tpmenu' value='yes'$checked
		onclick = 'return selfSubmit()'/>{$text['tp-compact']}</label>\n";

	# Option 6 - Focus on Generation - does not self-submit; has its own onchange event handler.
	$commonmenu .= "&nbsp;&nbsp; " . generateFocusControl($gfocus, $params['genstodo']);

##beta - Option 7 - a button for date/place options
if ($rrconfig['Text+UserDatePlace']) {
	$commonmenu .=  "<input type='hidden' name='dpoptions' id='dpoptions' value=''>\n";
	$commonmenu .= "&ensp; <span class='nw tpmenu rrtpPointer rrtphover'
		onclick='return toggleDatePlaceSubform();'>
		<img src='img/rrtextplus_down.gif'>{$text['tp-datesplaces']}</span>\n";
}
	# Place the Chart Content controls in the Inner Menu.
	$styleclass = $tngprint ? '' : " class='tpinnerline2'";
	$innermenu .= "<div$styleclass>$commonmenu</div>";

######### Third Inner Menu line: Page width control plus help & "Format for Printing" buttons
	# The Paper Width field's HTML code is generated by a structural function that
	# facilitates its use on the "main" page and on the Format for Printing page.
	$innermenu .= generatePageWidthField();
	#Add the chart help button
	$helplang = findhelp("rrtextplus_{$params['charttype']}_help.php");
	$innermenu .= "&ensp; <input type='button' name='b2' class='small' value=\"{$text['tp-charthelp']}\" onclick=\"return openTextPlusHelp('$helplang/rrtextplus_{$params['charttype']}_help.php');\">";
	#Add the printing help button
	$innermenu .= "&ensp; <input type='button' name='b2' class='small' value=\"{$text['tp-printinghelp']}\" onclick=\"return openTextPlusHelp('$helplang/rrtextplus_print_help.php');\">";
	#Add the Format for Printing button
	$innermenu .= "&ensp; <input type='button' name='b2' value=\"{$text['tp-formatforprint']}\"
		onclick=\"newwindow=window.open('{$_SERVER['REQUEST_URI']}&amp;tngprint=1',
		 'tngprint', 'width=850,height=600,status=yes,resizable=yes,scrollbars=yes'); newwindow.focus(); return false;\" rel='nofollow' title=''/>";

##### Print different menus for the Format for Printing page and the "main" page.
	# (Note that tng_menu returns an empty string in Format for Printing mode, so most programs don't need this if statement.
	if ($tngprint) {
		echo "<form id='form1' name='form1' class='tpmenu'>\n";
		echo generatePrintModeFormattingForm ($personID, $tree, $commonmenu);
		echo generateDatePlaceOptions();
	} else {
		echo getFORM( $params['phpBaseFilename'], '', 'form1', 'form1' );
		#Inner Mod Menu
		if (file_exists("rr_innermodmenu_lib.php")) {
			include_once('rr_innermodmenu_lib.php');
			$innermenu .= rrimmBuildInnerModMenu('TextPlus Charts', 1, 'pedconfig','other');
		}
		$innermenu .= generateDatePlaceOptions();
		echo tng_menu( "I", "pedigree", $personID, $innermenu);
	}
	echo "</form>\n";

	echo "<div class='tpmenu normal tpnoprint'>\n";
	#The link icons that go next to each person's name, which are omitted from the Format for Printing window
	echo generateIconLinksLegend('Anc');
	echo "</div>";
#Place of residence is not shown for people under 21 years old. They are assumed to be living with parents.
$yearsAgo21 = date('Y-m-d', strtotime('-21 years'));

##################### Start the chart #####################
#This inline style cannot be put in an external style sheet because PHP deterimines the font size.
echo "<div class='tpchart' style='font-family:arial,helvetica,sans-serif; margin-top:1ex;position:relative;font-size:{$fontsize}pt;'>\n";

#Start the chart with generation 1 and SOSA number 1
printAncestryPerson($personID, $result, 1, 1);

#Print a summary at the bottom of the chart
echo "<footer class='tpfooter rrtpBoxText'>{$text['tp-chartdisplayed']} {$count['Ancestors']} {$text['tp-directanc']}";
if ($count['Siblings'])
	echo " {$text['tp-among']} ", ($count['Ancestors']+$count['Siblings']), ' ', $text['tp-lcpeople'], ' ';
echo str_replace('XXX', $count['generations'], $text['tp-ingenerations']),"\n";
if ($count['leaflinks'])
	echo "<div>{$text['tp-additionalanc']}: {$count['leaflinks']}</div>\n";
#Report where ancestral lines began
if ($rrconfig['Text+AncestralLines']) {
	$ancestralLines = 0;
	$malehtml = $femalehtml = '';
	for ($g=2; $g<=$count['generations']; $g++) {
		if (isset($count['MaleLines'][$g]) && is_array($count['MaleLines'][$g])) {
			$thisgen = count($count['MaleLines'][$g]);
			$malehtml .= "<div class='rrLinesGen'>{$text['tp-gen']} $g: " . implode(', ', $count['MaleLines'][$g]) . "</div>\n";
			$ancestralLines += $thisgen;
		}
		if (isset($count['FemaleLines'][$g]) && is_array($count['FemaleLines'][$g])) {
			$thisgen = count($count['FemaleLines'][$g]);
			$femalehtml .= "<div class='rrLinesGen'>{$text['tp-gen']} $g: " . implode(', ', $count['FemaleLines'][$g]) . "</div>\n";
			$ancestralLines += $thisgen;
		}
	}
	if ($ancestralLines) {
		echo "<div class='tpnoprint tpLineEnd'>$ancestralLines {$text['tp-linesbegin']}.
		<a href='#' onclick=\"$('.tpLineEnd').toggle();return false;\">{$text['tp-showdetails']}</a></div>
		<div class='tpLineEnd' style='display:none;'>$ancestralLines {$text['tp-linesbegin']}
			<a href='#' onclick=\"$('.tpLineEnd').toggle();return false;\" class='tpnoprint'>{$text['tp-omitdetails']}</a>
		<div>{$text['male']} ({$text['tp-carrieddown']}):<br>$malehtml</div>
		<div>{$text['female']}:<br>$femalehtml</div>\n";
		if (count($count['CarriedIn']))
		echo "<div>{$text['tp-carriedin']}: " .
			count($count['CarriedIn']) . ': ' . implode(', ', $count['CarriedIn']) . "</div>\n";
		echo "</div>\n";
	}
} #End of ancestral lines data
echo "</footer\n";
if (!$tngconfig && $rrconfig['Text+AncFooter'])
	echo buildFloatingFooter();

echo "</div>\n"; #End div "tpchart"; end of the chart
?>
<script type="text/javascript" src="js/rpt_utils.js"></script>
<script type="text/javascript" src="js/rrtextplus.js"></script>
<script type="text/javascript">
//Inline code to initialize javascript variables used by a document.ready function
var genstodo = <?php echo $params['genstodo']; ?>;
if (genstodo == 99) genstodo=0;
var leaflinks = <?php echo $count['leaflinks']; ?>;
var gensdone = <?php echo $count['generations']; ?>;
var gensallowed = <?php echo $params['gensallowed']; ?>;
//Initialize a javascript variable for the initial generation to highlight in the chart.
var gfocus = <?php echo $gfocus; ?>;

///////////////////////////////////////////////////////////////////////
//////////////////////// document.ready function //////////////////////
$(function() {
<?php
	#Depending on a mod option, don't print the leaf link to a printer
	if ($rrconfig['Text+LeafLinkOmit'])
		echo "$('a.tpleaflink').addClass('tpnoprint');\n";
	#Make leaf links stand out more in the main window
	if (!$tngprint)
		echo "$('a.tpleaflink').addClass('databack');\n";

	#Another mod option determines whether to open the floating footer as the program loads
	if ($rrconfig['Text+AncFooter'])
		echo "openFloatingFooter();\n";
?>
	//Put the Highlight Generations options in their selection box
	$('#gfocus').append("<option value='0'><?php echo $text['none']; ?></option>");
	for (g=1; g<=gensdone; g++)
		$('#gfocus').append("<option value='"+g+"'>"+g+"</option>");
	$('#gfocus').val(gfocus); // Set selected option
	setFocus(0); //0 means don't scroll to the highlighted generation

	///////////////////////////////////////////////////////////////////////
	// Define the onlick handler for the location tracker icon, which creates the pop-up location tracker.
	// The wrapper elements for the popup box are created here (by a jQuery append).
	// The popup banner, close button, and text are supplied by rrtextplus_ancestor_tracker_ajx.php.
	// The trail attribute of the icon's <img> tag is a SOSA value.
	$('.rrtpTracker').click(function() {
		$('#trackdiv1').remove();  //Erase any other tracking popups
		var trail=$(this).attr('trail');
		//Place the new element at the beginning of the div that contains the icon;
		//that is, the div that wraps the first line of the person's data.
		var element = $(this).closest('div');
		if (!element.length)
			element = $(this).closest('div');
		element.prepend("<div id='trackdiv1'></div>");
		$.ajax({
			url: 'rrtextplus_ancestor_tracker_ajx.php',
			data: {tree:'<?php echo $tree; ?>',trail:trail, startingpersonID:'<?php echo $startingpersonID; ?>'},
			dataType: 'html',
			success: function(req){
				$('#trackdiv1').html(req);
				$('#trackbanner').addClass('fieldnameback fieldname').show();
				$('#tracktext').addClass('databack');
				//Activate the close button that we just created
				$('#trackdiv1 .rrtpCloseX').click(function() {
					$('#trackdiv1').remove();
				});
			}
		}); //End of AJAX call
	}); //End of tracker icon click event handler
}); //End of document.ready

/////////////////////////////////////////////////////////////////////
//OnClick event for the Show Generation #s and Show Siblings checkboxes.
//On the "main" page, generate Javascript code to submit the form.
//On the Format for Printing page, generate Javascript code as follows:
//   If both the "Show Gen #s" "Show Siblings" boxes are checked, then
//      Enable "Show Sibling #s" and erase the popup text.
//   Otherwise, Disable "Show Sibling #s" and put some pop-up text in the title attribute of the
//   label tag that encloses the "Show Sibling #s" checkbox..
function submitOrSetSiblingNumbers() {
<?php
	if (!$tngprint)
		echo "$('#form1').submit();\n";
	else
		echo "if ($('#showgennums').is(':checked') && $('#siblings').is(':checked')) {
			$('#siblingnums').attr('disabled', false);
			$('#siblingslabel').attr('title','');
			$('#siblingslabel').removeClass('disabledlabel');
		} else {
			$('#siblingnums').attr('disabled', true);
			$('#siblingslabel').attr('title',\"<?php echo {$text['tp-gennumrequired']}; ?>\");
			$('#siblingslabel').addClass('disabledlabel');
		}\n";
?>
} //end function submitOrSetSiblingNumbers

////////////////////////////////////////////////////////////////
//Function to highlight a generation. Usually called as an onclick function from the Focus on Generation field.
//The currently highlighted generation is in the global variable gfocus.
//gfocus is changed to the value in the Focus on Generation field.
function setFocus(onclick) {
	var newfocus = $('#gfocus').val();
	//Check for legal values of the new generation to focus on
	if (newfocus < 0) {
		$('#gfocus').val(0);
		return false;
	}
	if (newfocus > gensdone) {
		$('#gfocus').val(gensdone);
		return false;
	}
	// Clear the old highlighted generation
	if (gfocus) {
		$('.pdata.g'+(gfocus)+' a').removeClass('tphilite');
		$('.tpgnum.g'+(gfocus)).removeClass('tphilite');
		//$('.tpgnum.g'+(gfocus)).addClass('tpshrink');
	}
	gfocus = newfocus;
	//Highlight the new generation
	$('.pdata.g'+(gfocus)+' a').addClass('tphilite');
	//$('.tpgnum.g'+(gfocus)).removeClass('tpshrink');
	$('.tpgnum.g'+(gfocus)).addClass('tphilite');
	//Scroll the page to the first member of this generation
	if (onclick && newfocus) $('.tphilite').first().focus();
	return false;
} //end function setFocus

</script>
<?php
tng_footer( "" );

#########################################################################
#############  End of "main" program.  PHP Functions follow #############
#########################################################################
# The recursive function printAncestryPerson does the primary work of the program.
#It is called once by the main program to print the starting person.
#It then prints that father or mother, and calls itself to move up the tree.
# $result is a People table record result set. Depending on the "include siblings" run-time
#    option, it contains the ancestor, or the ancestor and all siblings.
# $sosa is this person's SOSA number
# $marriagestr is the marriage date/place for the person.
function printAncestryPerson($personID, $result, $generation, $sosa, $marriagestr='') {
	global $count, $params, $rrconfig, $vbars, $text;
	global $tree, $righttree;
	global $textplus_url, $getperson_url;
	global $families_table, $children_table;
	global $tngprint; #The pretty-print flag

	$thereIsNoFather = True;
	$thereIsNoMother = True;
	#Keep track of the maximum number of generations in the chart.
	if ($generation > $count['generations']) $count['generations'] = $generation;

	tng_data_seek($result,0);
	$row = tng_fetch_assoc($result);
	#In case we're doing siblings, remember which sibling is the ancestor
	$theAncestor = $row['personID'];
	$theAncestorSex = $row['sex'];
	$theAncestorName = getName($row);
	$theAncestorLastname = $row['lastname'];
	$theAncestorSex = $row['sex'];
	$theAncestorHyperlink = "<a href='$getperson_url" . "personID=$theAncestor&amp;tree=$tree'>$theAncestorLastname</a>";

	#Look for the parents' family record.
	#Get a family record set, and then one family record from that record set.
	### THIS PROGRAM CAN ONLY HANDLE ONE SET OF PARENTS. IT ALWAYS TAKES THE FIRST ONE
	$familyrow = null;
	$familyquery = "SELECT husband, wife, marrdate, marrdatetr, marrplace, living, private, branch, f.familyID
		FROM $children_table c
		INNER JOIN $families_table f ON c.familyID=f.familyID and c.gedcom=f.gedcom
		WHERE personID='$personID' AND c.gedcom='$tree'";
	$familyresult = tng_query($familyquery);
	$pmarriagestr = ''; #Parents' marriage data
	if ($familyresult)
		$familyrow = tng_fetch_assoc($familyresult);
	else $familyrow = '';

	if ($familyrow && $rrconfig['Text+ShowMarriage'] && $rrconfig['Text+AncestorMarriage']) {
		$frights = determineLivingPrivateRights($familyrow, $righttree);
		if ($frights['both']) {
			if ($rrconfig['Text+Date'])
				$pmarriagestr = rrGetDisplayDate($familyrow['marrdate'], $familyrow['marrdatetr']);
			if (strcasecmp($pmarriagestr,$text['tp-unknownvalue'])==0) $pmarriagestr="";
			if ($rrconfig['Text+Place'] && $familyrow["marrplace"])
				$pmarriagestr .= ($pmarriagestr ? " " : "") . $familyrow["marrplace"];
			if ($pmarriagestr) {
				$pmarriagestr = "{$params['marriageprefix']} $pmarriagestr";
				if (!$rrconfig['Text+3Lines']) $pmarriagestr = "($pmarriagestr)";
			}
		}
	}

	#Print the father unless we've reached the generation limit
	if ($familyrow && $generation < $params['genstodo'] ) {
		$parentID = $familyrow["husband"];
		if ($parentID) {
			$presult = getPersonData($tree, $parentID);
			if ($presult) {
				#There appears to be a parent
				$parentrow = tng_fetch_assoc($presult);
				$prights = determineLivingPrivateRights($parentrow, $righttree);
				$parentrow['allow_living'] = $prights['living'];
				$parentrow['allow_private'] = $prights['private'];
				$parentName = getName( $parentrow );
				if ($parentName != '') {
					#The father appears to be real
					$thereIsNoFather = False;
					printAncestryPerson($parentID, $presult, $generation+1, $sosa*2,$pmarriagestr);
				}
			}
		}
	}
	$vbars[$generation] = $theAncestorSex=="M" ? 0 : 1;
	#If we're printing a male without a father, then print a blank line to aid readability
	if (!$params['compact'] && $theAncestorSex == "M" && $thereIsNoFather && $count['BlankLines']) {
		PrintAncestryVbars($generation, 'blankfather');
		echo "</div>\n";
		$count['BlankLines']++;
	}
	### Print this person & possibly siblings
	if ($familyrow && $params['siblings']) {
		#Replace the one person with that person and siblings.
		$result = getChildrenData($tree, $row['famc']);
	} else {
		#Set the one-row dataset, back to the beginning
		tng_data_seek($result,0);
	}
	$childcount = 0;
	while($row = tng_fetch_assoc($result)) {
		$childcount++;
		$srights = determineLivingPrivateRights($row, $righttree);
		$row['allow_living'] = $srights['living'];
		$row['allow_private'] = $srights['private'];
		$fullname = getName( $row );

		#Indent as necessary.
		if ($row['personID'] == $theAncestor) {
			if ($row['sex'] == "M") $vbars[$generation]=3;
			else $vbars[$generation] = 2;
			$spaceleft = PrintAncestryVbars($generation,"ancestor");
		} else $spaceleft = PrintAncestryVbars($generation); //Not the ancestor

		if (!$spaceleft) return;

		#$count this person in the appropriate counter.
		if ($row['personID'] == $theAncestor)
			if ($generation>1) $count['Ancestors']++;
		else $count['Siblings']++;

		#Don't always hyperlink the name.
		if (false && $tngprint)
			$namelink = $fullname;
		else
			$namelink = "<a href='$getperson_url" . "personID={$row['personID']}&amp;tree=$tree'>$fullname</a>";
		#Add possible ancestor chart, 'locate person', and descendant chart links
		if ($generation>1) {
			$icons = generateIconLinks($row, 'Anc', ($row['personID'] == $theAncestor) ?$sosa : '');
			$spaceleft -= substr_count($icons,'<img')*2.3;
			$namelink .= $icons;
		}
		if ($srights['both'])
			$birthdeath = getVitalDates( $row );
		else $birthdeath = "";

		#At the limit of generations to print, look for more ancestors to link to.
		if ($row['personID'] == $theAncestor && $generation == $params['genstodo'] && $familyrow) {
			$count['leaflinks']++;
			$leaflink = "<a href='{$textplus_url}personID={$row['personID']}&amp;tree=$tree' class='tpleaflink' title=\"{$text['popupnote3']}\">&nbsp;=></a>";
		} else $leaflink="";

		$datastr = '';
		#Show generation numbers conditionally.  They are expanded and shrunk with the generation focus.
		if ($params['showgennums']) {
			$gennum = PrintGenerationNumbers2($generation, $childcount, $params['siblingnums']);
			$datastr .= "<span class='tpgnum g$generation'>&thinsp;$gennum</span>";
			$spaceleft -= strlen($gennum);
		}

		$datestring = $birthdeath;
		$mstr = '';
		#Option: Print the ancestor's marriage data only for the ancestor, not the descendant child.
		if($row['personID'] == $theAncestor) {
			if ($rrconfig['Text+AncestorMarriage']=='child' && $pmarriagestr)
				$mstr = $pmarriagestr;
			elseif ($rrconfig['Text+AncestorMarriage']=='parents' && $marriagestr)
				$mstr = $marriagestr;
		}
		if ($mstr) {
			if ($rrconfig['Text+3Lines']) $datestring .= "||$mstr";
			else $datestring .= $mstr;
		}

		#The person's name is expanded and shrunk with the generation focus.
		echo $datastr, "<span class='pdata g$generation' title='{$text['tp-gen']}", ($generation+1), "'>&thinsp;", $namelink, '&thinsp;';
		wrapdata ($spaceleft, $generation, strlen($fullname), $datestring, $leaflink, "person");
	} #loop for each child in the family

	#Clear the vertical bars for all generations above this one.
	for ($i=$generation+2; $i<=$count['generations']; $i++)
		$vbars[$i] = 0;

	#Print the mother unless we've reached the generation limit
	if ($familyrow && $generation < $params['genstodo'] ) {
		$parentID = $familyrow["wife"];
		if ($parentID) {
			$presult = getPersonData($tree, $parentID);
			if ($presult) {
				#There appears to be a parent
				$parentrow = tng_fetch_assoc($presult);
				$prights = determineLivingPrivateRights($parentrow, $righttree);
				$parentrow['allow_living'] = $srights['living'];
				$parentrow['allow_private'] = $srights['private'];
				$parentName = getName( $parentrow );
				if ($parentName != '' ) {
					#The mother appears to be real
					$thereIsNoMother = False;
					printAncestryPerson($parentID, $presult, $generation+1, $sosa*2+1, $pmarriagestr);
				}
			}
		}
	}
	#If we printed a female and there is no mother, print a blank line for readability
	if (!$params['compact'] && $theAncestorSex == "F" && $thereIsNoMother) {
		PrintAncestryVbars($generation, 'blankfem');
		echo "</div>\n";
		$count['BlankLines'] += 1;
	}

	#Count lines that begin here.
	if ($leaflink) {
		$count['CarriedIn'][] = $theAncestorHyperlink;
	} elseif ($thereIsNoMother && $thereIsNoFather) {
		if (!empty($theAncestorLastname) &&
			$theAncestorLastname != $text['tp-unknownvalue'] &&
			strpos('?? (', $theAncestorLastname) === false &&
			substr($theAncestorLastname, 0, 1) != '(' ) {
			if ($theAncestorSex == 'M')
				$count['MaleLines'][$generation][] = $theAncestorHyperlink;
			else $count['FemaleLines'][$generation][] = $theAncestorHyperlink;
		}
	}
} #end function printAncestryPerson

######################################################################
# Print the indentation (don't return it).
#Return the number of characters there is room for after the vbar.
#$what can contain "ancestor" and/or "wrap".
function PrintAncestryVbars($generation, $what="") {
	global $vbars, $count, $draw, $drawvbar, $params, $text;
	$count['Lines'] = $count['Lines'] + 1;

	$indenttext = "";
	$charactersused = 0;
	if ($generation>1) {
		#This loop prints indentation for all but the current generation.
		for ($i = 1; $i<$generation; $i++) {
			//# of chars in each generation = column width
			$indenttext .= "<span title='{$text['tp-gen']} $i'>" . str_repeat("&nbsp;",$params['indentation'] - 3) . $drawvbar[$vbars[$i]] . str_repeat("&nbsp;",2) . '</span>';
		}
		$indenttext .= "<span title='{$text['tp-gen']} $generation'>" . str_repeat("&nbsp;",$params['indentation'] - 3);
		if (strpos($what,"ancestor") !== false) {
			$indenttext .= $drawvbar[$vbars[$generation]] . $draw['hbar'];
			$vbars[$generation] = $vbars[$generation]-2;
		}
		else $indenttext .= $drawvbar[$vbars[$generation]] . $draw['hspaces'];
		$indenttext .= '</span>';
		$charactersused += ($generation-1)*$params['indentation'] + 3;
		if (0 && strpos($what,"blank") !== false) { #[[xyz]]
			$lastvbar = strrpos($indenttext, $draw['vbarOn']);
			if ($lastvbar !== false)
				$indenttext = substr($indenttext, 0, $lastvbar) . '' . substr($indenttext, $lastvbar+strlen($draw['vbarOn']));
		}
	} else $vbars[$generation] = 0;
	if (strpos($what,'wrap') === false) {
#[[xyz]]		$lineheight = (strpos($what, 'blank') !== false) ? " style='line-height:0.8ex;'" : '';
		$start = "<div>"; //The first line of a person's data
	} else {
		$start = '<br/>'; //A wrapped line
		$indenttext .= '&nbsp;&nbsp;&nbsp;&nbsp;';
		$charactersused += 4;
	}

	//To calculate the number of characters left, we find the inches used by the fixed font
	//indentation, and then determine the number of characters of proportional font that is
	//available, and then add the "widthplus" fudge factor.
	$inchesused = $charactersused/$params['fixedCPI'];
	$charactersleft = floor(($params['pagewidth']-$inchesused)*$params['proportionalCPI'] + $params['widthplus']);
	#If there is room for fewer than 10 characters, then print an error message
	if ($charactersleft<10) {
		$charactersleft=999;
		echo "$start<span style='font-weight:bold;color:red'>{$text['tp-nofit']} $generation: </span>\n";
	} else echo "$start<span class='tpindents'>$indenttext</span>";
	return $charactersleft;
} #end function PrintAncestryVbars

######################################################################
# This wrapping function is similar to the Descendant Text+ wrapping function, but that one
#has two data string arguments - one for birth/death, and the other for marriage.
#(Now that I've added marriage to this program, maybe I should add a marriage string argument.)
function wrapData( $spaceleft, $gen, $alreadydisplayedlength, $datastr, $leaflink, $what="") {
	global $rrconfig;

	$debug=false;
	$wrappedlinespaceleft = $spaceleft - (strpos($what, "wrap")===false ? 1 : 0);
	$spaceleft = $spaceleft - $alreadydisplayedlength;

	#If just printing the name has already exceeded the space available, start a new line.
	if ($spaceleft<=0) $spaceleft = PrintAncestryVbars($gen,0, "wrap");

	if (strpos($datastr, '||') !== false) {
		#Print data elements on separate lines. Don't worry about the available space.
		#There are possible data elements - age, birth, death, marriage
		$datalines = explode('||', $datastr);
		if ($datalines[0]) 
			echo $datalines[0], "</span></div>\n"; #age
		for ($l=1; $l<count($datalines)-1; $l++) {
			$spaceleft = PrintAncestryVbars($gen,0, "wrap");
			echo '&emsp;', $datalines[$l], "</span></div>\n";
		}
		$spaceleft = PrintAncestryVbars($gen,0, "wrap");
		echo '&emsp;', $datalines[count($datalines)-1];
		if ($leaflink) echo str_repeat('&ensp;',5),$leaflink;
		echo "</span></div>\n";
		return; ############ Early return ##########
	}
	$strlen1 = strlen($datastr) + ($leaflink ? 2 : 0);
	$spaceleftafter = $spaceleft - $strlen1;
	if ($spaceleftafter > 0) {
		//The data fits on the current line.
		#postpone...echo $datastr;
		if ($leaflink) {
			#Provide some white space between the data and the continuation link
			if ($spaceleftafter > 5) $datastr .= str_repeat('&ensp;',5);
			$datastr .= $leaflink;
		}
		echo $datastr,"</span></div>\n";
		return; ############ Early return ##########
	}
	#Some wrapping is necessary.
	#Consecutive )( characters signify the start of marriage data.
	$parts = explode(')(', $datastr, 2);
	if (count($parts)>1) {
		#There is a marriage date
		$lifespan = $parts[0] . ')';
		$marriage = '(' . $parts[1];
		$lifespanlen = strlen($lifespan);
		$marriagelen = strlen($marriage) + ($leaflink ? 2 : 0);
		if ($lifespanlen <= $spaceleft && $marriagelen <= $wrappedlinespaceleft) {
			#The lifespan fits on the rest of this line, and the marriage fits on the second
			echo $lifespan,"</span></div>\n";
			$spaceleft = PrintAncestryVbars($gen,0, "wrap");
			if ($leaflink) {
				if ($spaceleft > 5) $marriage .= str_repeat('&ensp;',5);
				$marriage .= $leaflink;
			}
			echo $marriage,"</span></div>\n";
			return; ############ Early return ##########
		}
	}

	#Break the data as necessary to print it all.
	#Note that breakLine lops off the return value from the beginning of $mystring, and modifies $spaceleft.
	$mystring = $datastr;
	$newtext = "";
	while ( $spaceleft < strlen($mystring) ) {
		echo breakLine($mystring, $spaceleft, $wrappedlinespaceleft);
		$spaceleft = PrintAncestryVbars($gen, "wrap");
	}
	$spaceleftafter = $spaceleft - strlen($mystring);
	# postpone... echo $mystring;
	#Position the continuation link near the right margin
	if ($leaflink) {
		if ($spaceleftafter > 5) $mystring .= str_repeat('&ensp;',5);
		$mystring .= $leaflink;
	}
	echo $mystring."</span></div>\n";
	return;
} #end function wrapData

############################################################################
# Generate the HTML code for the Focus on Generation selection box in the Ancestor Chart.
# The selection box options will defined after the report is generated so they can
# be limited by the number of generations in the chart.
function generateFocusControl() {
	global $text;
	$html = "<span class='nw'>{$text['tp-genfocus']}<select name='gfocus' id='gfocus' class='verysmall'
		onchange='return setFocus(1);'/></span>\n";
	return $html . "</select>\n";
} #end function generateFocusControl

?>
