<?php
##### rrtextplus_descend.php, installed by ###TextPlus Charts, by Robin Richmond
/* This is a variant of the TNG DescendText program.
This program draws lines from parents to descendants, and displays places well as dates,
like the TNG PDF Descendant Chart. It displays place of residence for living people,
and displays marriage date and place.
It displays alternate birth and death date and place when it there is no birth date, birth place,
death date, or death place.
It also goes to a lot of trouble to try to wrap text to a print margin.

(I have also implemented a separate mod (not yet published) that abbreviates place names,
and thus reduces the need to wrap the data for a person to multiple lines.
See the Wiki article on TextPlus Charts, and the TextPlus help file for more information.)

************************************************************/
# These parameters help allow the two Text+ program to share a lot of code. They should probably be constants.
$params['charttype'] = 'descend';
$params['phpBaseFilename'] = 'rrtextplus_' . $params['charttype'];
$params['phpFilename'] = $params['phpBaseFilename'] . '.php';
$textplus_dummy_parameter = ''; #dummy mod parameter that links to this mod's options, in a Mod Settings Block.
$tngModNames['TextPlus Charts'][$params['phpFilename']] = '17beta1'; #Register this mod
########################################################

$textpart = "pedigree"; #Gets the "pedigree" section of the language $text variables.
include("tng_begin.php");
include($subroot . "pedconfig.php");
#Read custom language strings from a mods subfolder rather than from regular cust_text.php files.
$modfolder = $text['tp-modfolder'];
if (file_exists("$modspath/$modfolder/{$mylanguage}_cust_text.php"))
	include("$modspath/$modfolder/{$mylanguage}_cust_text.php");
else include("$modspath/$modfolder/languages/English_cust_text.php");

#PHP functions installed by the Text Plus Charts mod common to both Text+ Chart Programs
include("{$cms['tngpath']}rrtextplus_lib.php");

###Call Graceful Exit if the mod has been installed.
if (file_exists('rrgracefulexit.php')) {
	include_once('rrgracefulexit.php');
	rrGracefulExit();
}
# Get the URLs of programs that link to the other charts in the inner menu.
$getperson_url = getURL( "getperson", 1 );
$descend_url = getURL( "descend", 1 );
$descendtext_url = getURL( "descendtext", 1 );
$textplus_url = getURL( $params['phpBaseFilename'], 1 );
$register_url = getURL( "register", 1 );
$pdfform_url = getURL( "rpt_pdfform", 1 );
#The "descendant to this point" chart program, linked to in the body of the report, not in the menu.
$desctracker_url = getURL( "desctracker", 1 );

$divctr = 1;
if( $pedigree['stdesc'] ) {
	$display = "none";
	$excolimg = "rrtextplus_plus";
	$imgtitle = $text['expand'];
}
else {
	$display = "block";
	$excolimg = "tng_minus";
	$imgtitle = $text['collapse'];
}

$level = 1;
$key = $personID;

$result = getPersonFullPlusDates($tree, $personID);
if( $result ) {
	$row = tng_fetch_assoc( $result );
	$righttree = checktree($tree);
	$rightbranch = $righttree ? checkbranch($row['branch']) : false;
	$rights = determineLivingPrivateRights($row, $righttree, $rightbranch);
	$row['allow_living'] = $rights['living'];
	$row['allow_private'] = $rights['private'];
	$namestr = getName( $row );
if (count($row)<30) var_dump($row);
	$logname = $tngconfig['nnpriv'] && $row['private'] ? $admtext['text_private'] : ($nonames && $row['living'] ? $text['living'] : $namestr);
}

$treeResult = getTreeSimple($tree);
$treerow = tng_fetch_assoc($treeResult);
$disallowgedcreate = $treerow['disallowgedcreate'];
$allowpdf = !$treerow['disallowpdf'] || ($allow_pdf && $rightbranch);
tng_free_result($treeResult);

$bookmarkstring = "<a href='{$textplus_url}personID=$personID&amp;tree=$tree&amp;generations=$generations'>Text+ {$text['tp-'.$params['charttype']]} $namestr ($personID)</a> $generations {$text['generations']}";
writelog($bookmarkstring);
preparebookmark($bookmarkstring);

$flags['tabs'] = $tngconfig['tabs'];
$flags['scripting'] = "<script type='text/javascript'>var tnglitbox;</script>\n";

tng_header( "{$text['tp-'.$params['charttype']]} $namestr", $flags );

#TextPlus CSS
echo "<link href='css/rrtextplus.css' rel='stylesheet' type='text/css'>\n";

$photostr = showSmallPhoto( $personID, $namestr, $rights['both'], 0, false, $row['sex'] );
echo tng_DrawHeading( $photostr, "{$text['tp-'.$params['charttype']]} <span class='nw'>$namestr</span>", getYears( $row ) );

#Check expected querystring parameters for values.
if (!isset($markasliving)) $markasliving=0;
if (!isset($hidedescend)) $hidedescend=0;
if (!isset($showgennums)) $showgennums=0;
if (!isset($siblingnums)) $siblingnums=0;
if ($markasliving) $allow_living = 0;
#Initialize the program formatting parameters.
initializeParameters($params['charttype']);
$params['markasliving'] = $markasliving;
$params['hidedescend'] = $hidedescend;
$params['showgennums'] = $showgennums;
$params['siblingnums'] = $siblingnums;

# $vbars - a 30 element control array (accommodating 30 generations), where each value can be 0, 1, 2, or 3
#          to indicate which character in $drawvbar should be printed for that generation.
$vbars = array_fill(0,30,1);
# $sbars - A 30 element control array similar to $vbars, where each character is 1 or 0, and
#          indicates whether to print the colon that draw a dotted vertical line to subsequent
#          spouses of a descendant.
$sbars = array_fill(0,30,0);

$count = array('generations'=>0, 'people'=>0, 'dependants'=>0, 'spouses'=>0, 'leaflinks'=>0,
	'lines'=>0, 'linesprinted'=>0, 'wrapped'=>0);
### End of initialization

#The number of Generations processed by one chart in the Inner Menu is normally passed to
#the other charts on the same Inner Menu (i.e. Ancestors vs Descendants). But the Text+ Charts
#can handle more generations than most of the other charts, so the Text+ Chart programs
#must limit the Generations parameter they pass to the other chart programs.
#For the Text+ Descendant Chart, generation limits are imposed this way:
# - The PDF chart and the two variants of $descend.php (Standard and compact) will be given the
#   value of the TNG system variable for "Initial Generations"; $pedigree['initdescgens']
# - The remaining charts (text, register, etc.) will be given the number of generations
#   specified by this program - but limited by the TNG system variable for the maximum
#   number of generations ($pedigree['maxdesc']).
$tnginitgen = $pedigree['initdescgens'] ? $pedigree['initdescgens'] : 12;
$tngmaxgen = $pedigree['maxdesc'] ? $pedigree['maxdesc'] : 4;
if ($generations && $generations< $tngmaxgen) $linkgens = $generations;
else $linkgens = $tngmaxgen;

if (!isset($parentset)) $parentset='';
######### Inner Menu line 1
	$innermenu = "<input type='hidden' name='personID' value='$personID'>";
	$innermenu .= "<input type='hidden' name='tree' value='$tree'>";
	$innermenu .= "<div>";
	$innermenu .= generateChartTypeLabel($params['charttype']);

	$innermenu .= "<a href=\"$descend_url" . "personID=$personID&amp;tree=$tree&amp;display=standard&amp;generations=$tnginitgen\" class=\"lightlink\">{$text['pedstandard']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$descend_url" . "personID=$personID&amp;tree=$tree&amp;display=compact&amp;generations=$tnginitgen\" class=\"lightlink\">{$text['pedcompact']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	$innermenu .= "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree&amp;generations=$linkgens\" class=\"lightlink\">{$text['pedtextonly']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";

	#The new TextPlus link always follows the Text version of the chart in the inner menu
	$innermenu .= "<a href='{$textplus_url}personID=$personID&amp;tree=$tree&amp;parentset=$parentset&amp;generations=$linkgens' class='lightlink3' id='textlnk\'>{$text['tp-textplus']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";

	#If links.php has been installed by the mod "Male and Female Descendant and Parent Ancestor Lines",
	#do not use it (it's an Ancestor chart), but do link to the male and female descendant lines
	#charts that the mod has implemented by modifying descendtext.php.
	if (file_exists('lines.php')) {
		if (empty($text['malelines'])) $text['malelines'] = 'Male Lines';
		$innermenu .= "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree&amp;displayoption=male&amp;generations=$linkgens\" class=\"lightlink\">{$text['malelines']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
		if (!$text['femalelines']) $text['femalelines'] = 'Female Lines';
		$innermenu .= "<a href=\"$descendtext_url" . "personID=$personID&amp;tree=$tree&amp;displayoption=female&amp;generations=$linkgens\" class=\"lightlink\">{$text['femalelines']}</a> &nbsp;&nbsp; | &nbsp;&nbsp; \n";
	}

	$innermenu .= "<a href=\"$register_url" . "personID=$personID&amp;tree=$tree&amp;generations=$linkgens\" class=\"lightlink\">{$text['regformat']}</a>\n";
	if($allowpdf)
		$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href=\"#\" class=\"lightlink\" onclick=\"tnglitbox = new LITBox('$pdfform_url" . "pdftype=desc&amp;personID=$personID&amp;tree=$tree&amp;generations=$tnginitgen',{width:350,height:350});return false;\">PDF</a>\n";

	#Make sure that the Count Descendants program exists before trying to link to it.
	if (file_exists('rrcount_descendants.php')) {
		if (empty($text['rrcd-descendcount'])) $text['rrcd-descendcount'] = 'Count';
		$innermenu .= " &nbsp;&nbsp; | &nbsp;&nbsp; <a href='rrcount_descendants.php?personID=$personID&amp;tree=$tree' class='lightlink'>{$text['rrcd-descendcount']}</a>\n";
	}
	$innermenu .= "</div>\n";

######### Second Inner Menu line, with Chart Content options
	#These controls are placed in the main window Inner Menu and in the Format for Printing window
	#Option 1 - Generations selection box, which is built by a function because it is used
	#in both Text+ Chart programs.
	$commonmenu = generateGenerationsSelectionBox($params['gensallowed'], $params['genstodo']);

	#Option 2 - 'Hide Living' is meaningful only if the user is logged in, and is omitted
	#completely if the user is not logged in.
	if ($currentuser) {
		$checked = $markasliving ? ' checked' : '';
		$commonmenu .= "&nbsp; <label class='nw' title=\"{$text['tp-likenotlogged']}\">
			<input type='checkbox' class='tpmenu' name='markasliving' id='markasliving' value='yes'$checked
				onclick='return submitOrSetTheirDescendants();'
				title='{$text['tp-likenotlogged']}'>{$text['tp-hideliving']}</label>...";
	}

	#Option 3 - Hide descendants of 'Living' people.
	#If the user is logged in, this field is disabled unless the "Mark as living" box is checked.
	#If the user is not logged in, this field is never disabled.
	#The label for this checkbox (in $prompt) depends on whether the user is logged in.
	if ($currentuser && !$markasliving)
		{ $disabled = ' disabled'; $disabledclass=" class='disabledlabel'"; }
	else $disabled = $disabledclass = '';
	$checked = $params['hidedescend'] ? ' checked' : '';
	if ($currentuser) {
		$prompt = $text['tp-anddescend'];
		$titletext = $text['tp-hidedescend'];
	} else {
		$prompt = $text['tp-hidedescend'];
		$titletext = '';
	}
	$commonmenu .="&nbsp; <label class='nw' title=\"$titletext\" id='hidedesclabel'$disabledclass>
		<input type='checkbox' class='tpmenu' name='hidedescend' id='hidedescend' value='yes'
		onclick = 'return selfSubmit();'$checked$disabled/><span id='hidedesclabel'$disabledclass>$prompt</span></label>\n";

	# Option 4 - Display Generation #
	#When this checkbox is not checked, the show sibling #'s checkbox is disabled and a warning
	#message is saved to the the label element that wraps the show sibling # checkbox.
	$checked = $showgennums ? ' checked' : '';
	$commonmenu .= "&nbsp; <label class='nw'>
		<input type='checkbox' class='tpmenu' name='showgennums' id='showgennums' value='yes'$checked
			onclick = 'return submitOrSetSiblingNumbers()'/>{$text['tp-gennums']}</label>\n";

	### Option 5 - Display Sibling count - Is enabled only if Generation #s is checked.
	#If it is disabled, a pop-up message is generated in the title attribute of the enclosing <label> tag.
	$checked = $siblingnums ? ' checked' : '';
	if ($showgennums) {
		$siblingnumsmessage = $disabled = $disabledclass = '';
	} else {
		$disabled = ' disabled';
		$siblingnumsmessage = $text['tp-gennumrequired'];
		$disabledclass = ' disabledlabel';
	}
	$commonmenu .="&nbsp; <label class='nw$disabledclass' id='siblingslabel' title=\"$siblingnumsmessage\">
		<input type='checkbox' name='siblingnums' id='siblingnums' class='tpmenu' value='yes'$checked$disabled
			onclick = 'return selfSubmit();'>{$text['tp-siblingnums']}</label>\n";

##beta - A button for date/place options - but only in the Format for Printing page
if ($rrconfig['Text+UserDatePlace']) {
	$commonmenu .=  "<input type='hidden' name='dpoptions' id='dpoptions' value=''>\n";
	$commonmenu .= "&ensp; <span class='nw tpmenu rrtpPointer rrtphover'
		onclick='return toggleDatePlaceSubform();'>
	<img src='img/rrtextplus_down.gif'>{$text['tp-datesplaces']}</span>\n";
}

	# Place the Chart Content controls in the Inner Menu.
	$styleclass = $tngprint ? '' : " class='tpinnerline2'";
	$innermenu .= "<div$styleclass>$commonmenu</div>";

######### Third Inner Menu line: Page width control plus help & "Format for Printing" buttons
	# The Paper Width field's HTML code is generated by a structural function that
	# facilitates its use on the "main" page and on the Format for Printing page.
	$innermenu .= generatePageWidthField();
	#Add the chart help button
	$helplang = findhelp("rrtextplus_{$params['charttype']}_help.php");
	$innermenu .= "&ensp; <input type='button' name='b2' class='small' value=\"{$text['tp-charthelp']}\" onclick=\"return openTextPlusHelp('$helplang/rrtextplus_{$params['charttype']}_help.php');\">";
	#Add the printing help button
	$innermenu .= "&ensp; <input type='button' name='b2' class='small' value=\"{$text['tp-printinghelp']}\" onclick=\"return openTextPlusHelp('$helplang/rrtextplus_print_help.php');\">";
	#Add the Format for Printing button
	$innermenu .= "&ensp; <input type='button' name='b2' value=\"{$text['tp-formatforprint']}\"
		onclick=\"newwindow=window.open('{$_SERVER['REQUEST_URI']}&amp;tngprint=1',
		 'tngprint', 'width=850,height=600,status=yes,resizable=yes,scrollbars=yes'); newwindow.focus(); return false;\" rel='nofollow' title=''/>";

##beta - a button for date/place options, here rather than in the "commonmenu"
##It WAS here; leave the code in case we return it to this position
if (false && $rrconfig['Text+UserDatePlace']) {
	$innermenu .= "<input type='hidden' name='dpoptions' id='dpoptions' value=''>\n";
	$innermenu .=  "&ensp; <span class='nw tpmenu rrtpPointer tphover'
		onclick='return toggleDatePlaceSubform();'>
		<img src='img/rrtextplus_down.gif'>{$text['tp-datesplaces']}</span>\n";
}
######### Print different menus for the Format for Printing page and the "main" page.
	# (Note that tng_menu returns an empty string in Format for Printing mode, so most programs don't need this if statement.
	if ($tngprint) {
		echo "<form id='form1' name='form1' class='tpmenu'>\n";
		echo generatePrintModeFormattingForm ($personID, $tree, $commonmenu);
		echo generateDatePlaceOptions();
	} else {
		echo getFORM( $params['phpBaseFilename'], '', 'form1', 'form1' );
		#Insert an Inner Mod Menu for admins
		if (file_exists("rr_innermodmenu_lib.php")) {
			include_once('rr_innermodmenu_lib.php');
			$innermenu .= rrimmBuildInnerModMenu('TextPlus Charts', 1, 'pedconfig','other');
		}
		$innermenu .= generateDatePlaceOptions();
		echo tng_menu( "I", "descend", $personID, $innermenu);
	}
	echo "</form>\n";

##### Below the inner menu, display the icon legend and links for the collapse/expand feature.
	echo "<div class='tpmenu normal tpnoprint'>\n";
	#The link icons that go next to each person's name, which are omitted from the Format for Printing window
	echo generateIconLinksLegend('Desc');
	#The expand and collapse icon explanations are the same in both windows.
	echo "<div style='line-height:2em;'><img src=\"{$cms['tngpath']}img/rrtextplus_plus.gif\" width='9' height='9' alt='' border='0'/> = {$text['expand']},
		<img src=\"{$cms['tngpath']}img/tng_minus.gif\" width='9' height='9' alt='' border='0'/> = {$text['collapse']}\n";
	echo "&nbsp; <a href='#' class='snlink' onclick=\"return toggleAll('');\">{$text['expandall']}</a>
		&nbsp; <a href='#' class='snlink' onclick=\"return toggleAll('none');\">{$text['collapseall']}</a></div>\n";
	echo "</div>";

#Place of residence is not shown for people under 21 years old. They are assumed to be living with parents.
$yearsAgo21 = date('Y-m-d', strtotime('-21 years'));
#List of all families seen so that we can recognize duplicates.
$families = array();

##################### Start the chart #####################
#This inline style cannot be put in an external style sheet because PHP deterimines the font size.
echo "<div class='tpchart' style='margin-top:1ex;position:relative;font-size:{$fontsize}pt;line-height:",$height*1.2,"%;'>\n";
### Read the parents' row, and get & display information about each parent
$parentsdisplayed = 0;
$parentsresult = getChildParentsFamilyMinimal($tree, $personID);
if ($parentsresult ) {
	$parentsrow = tng_fetch_assoc( $parentsresult  );
	$parents[0] = $parentsrow['husband'];
	$parents[1] = $parentsrow['wife'];
	for ($i=0;$i<=1;$i++) {
		$parentname = $parentnamestr = $parentdatastr = '';
		if( $parents[$i] ) {
			$parentsdisplayed++;
			$parentresult = getPersonData($tree, $parents[$i]);
			if( $parentresult ) {
				$parentrow = tng_fetch_assoc( $parentresult );
				$prights = determineLivingPrivateRights($parentrow, $righttree);
				$parentrow['allow_living'] = $prights['living'];
				$parentrow['allow_private'] = $prights['private'];
				$parentname = getName( $parentrow );
				//Don't hyperlink the parent name in print mode.
				if ($tngprint)
					$parentnamestr = $parentname;
				else $parentnamestr = " <a href='$getperson_url" . "personID={$parentrow['personID']}&amp;tree=$tree'>$parentname</a> ";

				//Don't get the birth/death data if we don't have rights to this person.
				if ($prights['both'])
					$parentdatastr = getVitalDates( $parentrow );
				//We have parent data to display
				switch ($parentrow['sex']) {
					case "M": $parenttype="father"; break;
					case "F": $parenttype="mother"; break;
					default: $parenttype="parent";
				}
				#Parent's name
				echo "<div><strong>{$text[$parenttype]}</strong>:$parentnamestr&thinsp;";
				#Parents can get descendant chart and ancestor chart icons
				echo generateIconLinks($parentrow, 'Desc');
				#The rest of the parent's data, which can wrap to the page margin
				echo "&thinsp;" . str_replace("||", "<br>", $parentdatastr), "</div>\n";
			} # end if there is a parent record
		} #end if the family record has a value for this parent
	} #end for each parent
} #end of are there parents

#Blank line if we displayed any parents.
if ($parentsdisplayed) echo "<br/>";

#Print the starting person's name and data. No wrapping is needed.
#Note that $namestr and $rights[] were defined in the page heading.
#There are no vbars to the left of the starting person, so we won't bother with wrapping.
#Don't hyperlink the name in the Format for Printing page, or if the name is 'Living' or 'Private'
if ($tngprint || !$rights['both'])
	echo "<strong>$namestr</strong>\n";
else echo "<strong><a href='$getperson_url" . "personID=$personID&amp;tree=$tree'>$namestr</a></strong> \n";
if ($rights['both']);
	echo str_replace('||', '<br> ', getVitalDates( $row )) . "\n";

$count["descendants"] = 1;
$count["linesprinted"] = $parentsdisplayed+1;

//Call the recursive function getIndividual to print the starting person person's spouses and descendants, etc.
echo getIndividual( $key, $row['sex'], $level + 1, $personID, $rights['both']);

#####################################################
#Print a fairly complicated summary at the bottom of the chart.
#The summary as read by the user isn't all that complex, but it has many parts that
#are separated into <span> elements because
#a. JavaScript code can change some elements of the description when branches of the descendancy tree are collapsed by the user, and
#b. some elements are displayed on the screen only (not printed).
#For instance, if branches have been collapsed, then a line is added that says "nn people have been hidden",
#and the phrase "Links to charts that show additional ancestors" is changed to "Visible links to charts...".
$count['people'] = $count['descendants']+$count['spouses']+$parentsdisplayed;
$count['wrapped'] = $count['linesprinted'] - $count['people'];

#The footer contains 3 seqential divs:
# 1. footerclose is the close element - just an X - for the fixed-position pop-up footer.
#    Its click function hides the banner div and itself, unboxes the footer and removes the background color.
# 2. footerbanner is the footer heading that says "Report summary".
#    It is displayed when a tree-pruning link is clicked, and closed with the close button.
# 3. footertext is the rest of the footer.
#    It normally sits at the bottom of the page, but when a tree-pruning link is clicked,
#    It is placed inside a box that floats at the bottom of the browser window.
#Build the content of the footer in a PHP variable so we can place it in the footer and in the floating footer.
#The footer contains span elements whose contents are updated as branches of the chart are pruned and restored.
#They are identified by classes rather than elements so that both the footer and floating footer are updated.
$html = "{$text['tp-inchart']}<span class='initially' style='display:none;'> ({$text['tp-initially']})</span>: {$count['people']} ({$text['tp-including']} {$count['spouses']} {$text['tp-spouses']}";
#Deal with parents by saying "and 2 parents" or "and 1 parent"
if ($parentsdisplayed) {
	if ($parentsdisplayed==1)
		$html .= ' ' . $text['tp-and1parent'];
	else $html .= ' ' . str_replace('XXX',$parentsdisplayed,$text['tp-andparents']);
}
#Number of generations
$html .= ') ' . str_replace('XXX', $count['generations']-1, $text['tp-ingenerations']) . '.';
#Statement about number of hidden people - initially not displayed.
$html .= "<div class='hiddentext' style='display:none;'>{$text['tp-hidden']}: <span class='nhidden'></span>, leaving <span class='nremaining'></span> {$text['tp-lcpeople']}.</div>\n";
#Statement about number of links to charts that show additional descendants
$html .= "<div><span class='linkstext'>{$text['tp-links']}</span> {$text['tp-additionaldesc']}: <span class='ndesclinks'>{$count['leaflinks']}</span></div>\n";
#Number of wrapped lines.
$html .= "<div class='cgmenu'><span class='linestext'>{$text['tp-lines']}</span> {$text['tp-wrapped']}: <span class='nwrapped'>{$count['wrapped']}</span></div>\n";
echo "<footer class='rrtpBoxText'>$html</footer>\n";

if ($rrconfig['Text+DescFooterAppears'] !== 'none')
	echo buildFloatingFooter();

echo "</div>\n"; #End div "tpchart"; end of the chart
?>

<script type="text/javascript" src="js/rpt_utils.js"></script>
<script type="text/javascript" src="js/rrtextplus.js"></script>
<script type="text/javascript">
//Initialize javascript variables used by a document.ready function that uses counts in the chart
//summary to set up the Generations selection box in the inner menu or Format for Printing form.
var genstodo = <?php echo $params['genstodo']; ?>;
if (genstodo == 99) genstodo=0;
var leaflinks = <?php echo $count['leaflinks']; ?>;
var gensdone = <?php echo $count['generations']; ?>;
var gensallowed = <?php echo $params['gensallowed']; ?>;
///////////////////////////////////////////////////////////////////////
//////////////////////// document.ready function //////////////////////
$(function() {
<?php
	#Depending on a mod option, don't print the leaf link to a printer
	if ($rrconfig['Text+LeafLinkOmit'])
		echo "$('a.tpleaflink').addClass('tpnoprint');\n";
	#Make leaf links stand out more in the main window
	if (!$tngprint)
		echo "$('a.tpleaflink').addClass('databack');\n";

	#Another mod option determines whether to open the floating footer as the program loads
	if ($rrconfig['Text+DescFooterAppears'] == 'load')
		echo "openFloatingFooter();\n";
?>
	//Apply cookies to trim the tree as it was the last time this patient's descendant chart was displayed
	cookieList = document.cookie.split(cookieName +'|');
	for (rr=1; rr<cookieList.length; rr++) {
		cookieParts =  cookieList[rr].split('|');
		if (cookieParts[0] == personID && cookieParts[1] == maxGenerations && cookieParts[4]=='hidden')
			toggleDescSection(cookieParts[2]);
	}
	///////////////////////////////////////////////////////////////////////
	//Define an Onclick event handler for the tree pruning boxes (plus and minus signs)
	//Show or hide the appropriate subtree, and possibly open the floating footer.
	$('.plusminus').click(function() {
		var divname = $(this).attr('id').replace('plusminus','');
		toggleDescSection(divname);
		//Display the floating footer
		// if it is hidden AND we are ever allowed to open it AND
		//  (EITHER (it hasn't ever been opened AND it should be opened when we first prune or restore it)
		//       OR (it was explicitly closed and are allowed to re-open it))
		if (footerstatus != 'open' && '<?php echo $rrconfig['Text+DescFooterAppears']; ?>'!='never'
			&& ((footerstatus=='never' &&'<?php echo $rrconfig['Text+DescFooterAppears']; ?>'=='prune')
			  || (footerstatus=='closed' && '<?php echo $rrconfig['Text+DescFooterReopens']; ?>'=='1')))
			openFloatingFooter();
		return false;
	});
	///////////////////////////////////////////////////////////////////////
	// Define the onlick handler for the location tracker icon, which creates the pop-up location tracker.
	// The wrapper elements for the popup box are created here (by a jQuery append).
	// The popup banner, close button, and text are supplied by rrtextplus_descend_tracker_ajx.php.
	// The tracking trail is in the 'trail' attribute of the icon's <img> tag.
	$('.rrtpTracker').click(function() {
		$('#trackdiv1').remove();  //Erase any other tracking popups
		var trail=$(this).attr('trail');
		//Place the new element at the beginning of the div that contains the icon;
		//that is, the div that wraps the first line of the person's data.
		var element = $(this).closest('div').next('div');
		if (!element.length)
			element = $(this).closest('div');
		element.prepend("<div id='trackdiv1'></div>");
		$.ajax({
			url: 'rrtextplus_descend_tracker_ajx.php',
			data: {tree:'<?php echo $tree; ?>',trail:trail},
			dataType: 'html',
			success: function(req){
				$('#trackdiv1').html(req);
				$('#trackbanner').addClass('fieldnameback fieldname').show();
				$('#tracktext').addClass('databack');
				//Activate the close button that we just created
				$('#trackdiv1 .rrtpCloseX').click(function() {
					$('#trackdiv1').remove();
				});
			}
		}); //End of AJAX call
	}); //End of tracker icon click event handler
}); //End of document.ready

///////////////////////////////////
var plusImages = Array();
plusImages['plus'] = "<?php echo $cms['tngpath'] ?>img/rrtextplus_plus.gif";
plusImages['minus'] = "<?php echo $cms['tngpath'] ?>img/tng_minus.gif";
//Three constants used to manage cookies that indicate which branches of the tree have been trimmed.
var tngDomain = "<?php echo $tngdomain; ?>";
var personID = "<?php echo $personID; ?>";
var maxGenerations = "<?php echo $generations; ?>";
var cookieName = 'tng_rrtextplus_descendtrimX';

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
// Javascript to handle checkboxes in the inner menu
/////////////////////////////////////////////////////////////////////
//OnClick event for the "Hide data for living and private people" checkbox.
//On the "main" page, submit the form. On the Format for Printing page, set the
//"Hide Descendants" disabled status to the inverse of of the "Hide Data" checked status
function submitOrSetTheirDescendants() {
<?php
	if (!$tngprint)
		echo "$('#form1').submit();\n";
	else {
		echo "$('#hidedescend').attr('disabled', !$('#markasliving').is(':checked'));
		if ($('#hidedescend').attr('disabled')) $('#hidedesclabel').addClass('disabledlabel');
		else $('#hidedesclabel').removeClass('disabledlabel');\n";
	}
?>
} //end function submitOrSetTheirDescendants

//OnClick event for the Show Generation #s checkbox.
//On the "main" page, generate Javascript code to submit the form.
//On the Format for Printing page, generate Javascript code as follows:
//   If the checked status of "Show generation #s" is checked,
//      Enable "Show sibling #s" and erase the pop-up text.
//   Otherwise, Disable "Show sibling #s" and put pop-up text in the <label> tag that wraps the checkbox.
function submitOrSetSiblingNumbers() {
<?php
	if (!$tngprint)
		echo "$('#form1').submit();\n";
	else
		echo "if ($('#showgennums').is(':checked')) {
			$('#siblingnums').attr('disabled', false);
			$('#siblingslabel').attr('title','');
			$('#siblingslabel').removeClass('disabledlabel');
		} else {
			$('#siblingnums').attr('disabled', true);
			$('#siblingslabel').attr('title',\"<?php echo {$text['tp-gennumrequired']}; ?>\");
			$('#siblingslabel').addClass('disabledlabel');
		}\n";
?>
} //end function submitOrSetSiblingNumbers

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// Javascript code that handles tree-pruning, including changes to the footer
// that describe the number of people who have been removed from the tree and
// that refer to visible continuation links rather than just continuation links.
///////////////////////////////////////////////////////////////////////
var collapsemsg = "<?php echo $text['collapse']; ?>";
var expandmsg = "<?php echo $text['expand']; ?>";
var wrappedLines = <?php echo $count['linesprinted']-$count['people']; ?>;
var wrappedLinesInitially = wrappedLines;
var descendantLinks = <?php echo $count['leaflinks']; ?>;
var descendantLinksInitially = descendantLinks;
var peopleTotal = "<?php echo $count['people']; ?>";
var peopleRemoved = 0;
//////////////////////////////////////////////////////////
//Function to toggle the display of a branch of the tree when the user clicks on a
//plus or minus sign, and to update the chart summary.
//Arguments: key is the ID of the branch that is being toggled.
function toggleDescSection(key) {
	var section = $('#' + key);
	if( section.css('display') == 'none' ) {
		//It's hidden, so display it.
		section.show();
		swap("plusminus" + key,"minus");
		//The next three variables are arrays of elements that are to be restored
		var collapsedLinesList = $('#' + key).find('span.tpindents'+key).removeClass('tpindents'+key).addClass('tpindents');
		var collapsedPeopleList = $('#' + key).find('.ppl'+key).removeClass('ppl'+key).addClass('ppl');
		var collapsedLinksList = $('#' + key).find('.tpleaflink'+key).removeClass('tpleaflink'+key).addClass('tpleaflink');
		wrappedLines += collapsedLinesList.length - collapsedPeopleList.length;
		peopleRemoved -= collapsedPeopleList.length;
		descendantLinks += collapsedLinksList.length;
		if (!peopleRemoved) {
			//Everyone has been redisplayed, so restore the initial display values
			$('.initially').hide(); //(initially) in first line of summary
			$('.hiddentext').hide(); //Number of people hidden
			$('.linkstext').html("<?php echo $text['tp-links']; ?>");
			$('.linestext').html("<?php echo $text['tp-lines']; ?>");
		}
		//Update the counts
		$('.nhidden').html(peopleRemoved);
		$('.nremaining').html(peopleTotal - peopleRemoved);
		$('.nwrapped').html(wrappedLines);
		$('.ndesclinks').html(descendantLinks);
		//Clear the "this descendant tree node has been trimmed" cookie.
		document.cookie = cookieName +'|'+personID +'|'+maxGenerations+'|' + key+"|=|;expires=18 Dec 2013;";
	}
	else { //Hide the section
		swap("plusminus" + key,"plus");
		section.hide();
		var collapsedLinesList = $('#'+key).find('span.tpindents').removeClass('tpindents').addClass('tpindents'+key);
		var collapsedPeopleList = $('#'+key).find('.ppl').removeClass('ppl').addClass('ppl'+key);
		var collapsedLinksList = $('#'+key).find('.tpleaflink').removeClass('tpleaflink').addClass('tpleaflink'+key);
		if (!peopleRemoved) {
			//This is the first removal, so display the alternate text
			$('.initially').show(); //(initially) in first line of summary
			$('.hiddentext').show(); //Number of people hidden
			$('.linkstext').html("<?php echo $text['tp-visiblelinks']; ?>");
			$('.linestext').html("<?php echo $text['tp-visiblelines']; ?>");
		}
		peopleRemoved += collapsedPeopleList.length;
		descendantLinks -= collapsedLinksList.length;
		wrappedLines -= collapsedLinesList.length - collapsedPeopleList.length;
		//Update the counts
		$('.nhidden').html(peopleRemoved);
		$('.nremaining').html(peopleTotal - peopleRemoved);
		$('.nwrapped').html(wrappedLines);
		$('.ndesclinks').html(descendantLinks);
		//Set a cookie that says this branch of this person's tree has been trimmed.
		document.cookie = cookieName +'|'+personID +'|'+ maxGenerations+'|' + key +'|=|hidden|;';
	}
	return false;
} //end function toggleDescSection

////////////////////////////////////////////////////////////////
//Function to open or collapse all of the branches of the descendancy tree.
//Input argument disp is '' for open, and 'none' for collapse.
function toggleAll(disp) {
	var i = 1;
	while (jQuery("#fc"+i).length) {
		jQuery("#fc"+i).css('display',disp);
		if( disp == '' )
			swap("plusminusfc" + i,"minus");
		else
			swap("plusminusfc" + i,"plus");
		i++;
	}
	return false;
} //end function toggleAll

////////////////////////////////////////////////////////////////
//Function to swap the little boxed plus and minus images that keep track of
//which branches of the descendancy tree are collapsed.
function swap(x, y) {
	$('#'+x).attr('title',y == "minus" ? collapsemsg : expandmsg);
	//document.images[x].src=eval(y+'.src');
	$('#'+x).attr('src',plusImages[y]);
} //end function swap

</script>
<?php
tng_footer( "" );

#########################################################################
#############  End of "main" program.  PHP Functions follow #############
#########################################################################
/** The recursive function getIndividual does the primary work of the program.
* It is called once by the main program to print the starting person.
* It then prints that person and spouse, and calls itself to print each child.
*/
function getIndividual( $key, $sex, $level, $trail, $prightsboth, $generationnumbers='' ) {
	global $rrconfig;
	global $tree, $text, $admtext, $cms, $getperson_url, $righttree;
	global $desctracker_url, $divctr, $display, $excolimg, $textplus_url, $imgtitle;
	global $params;  #Array of control parameters
	global $draw, $vbars, $sbars; #text-plus line-drawing strings
	global $tngprint; #Parameter that tells us that we're in the Format for Printing window.
	global $count;
	global $families; #List of all families seen so that we can recognize duplicates.
	$rval = '';
	$gennums = '';

	if ($level > $count["generations"]) $count["generations"]=$level;
	if( $sex == "M" ) {
		$self = "husband";
		$spouse = "wife";
		$spouseorder = "husborder";
	}
	else if( $sex == "F" ){
		$self = "wife";
		$spouse = "husband";
		$spouseorder = "wifeorder";
	}
	else {
		$self = $spouse = $spouseorder = "";
	}
	if( $spouse ) //If we know the spouse's gender
		$result = getSpouseFamilyDataPlusDates($tree, $self, $key, $spouseorder);
	elseif( $key ) //Use the person number and do a Union to find wives and husbands
		$result = getSpouseFamilyDataUnionPlusDates($tree, $key);
	$numberofmarriages = tng_num_rows($result);
	if( !$numberofmarriages && $key) {
		$result = getSpouseFamilyDataUnionPlusDates($tree, $key);
		$self = $spouse = $spouseorder = "";
	}

	if( $result ) {
		$numberofmarriages = tng_num_rows($result);
		$marriagecount = 0;
		$alreadyprinted = 0;
		while( $row = tng_fetch_assoc( $result ) ) {
			$marriagecount++;
			$mrights = determineLivingPrivateRights($row, $righttree);
			$mrow['allow_living'] = $mrights['living'];
			$mrow['allow_private'] = $mrights['private'];
			$spouserow = array();
			$spousename = $spousenamestr = $spousedatastr = $marriagestr = '';
			if( !$spouse )
				$spouse = $row['husband'] == $key ? "wife" : "husband";
			if( $row[$spouse] ) {
				$spouseresult = getPersonData($tree, $row[$spouse]);
				if( $spouseresult ) {
					$spouserow = tng_fetch_assoc( $spouseresult );
					$srights = determineLivingPrivateRights($spouserow, $righttree);
					$spouserow['allow_living'] = $srights['living'];
					$spouserow['allow_private'] = $srights['private'];
					$spousename = getName( $spouserow );
					$marriagestr = '';
					$count["spouses"]++;

					#Don't always hyperlink the name.
					if ($tngprint)
						$spousenamestr = $spousename;
					else
						$spousenamestr = "<a href='$getperson_url" . "personID={$spouserow['personID']}&amp;tree=$tree'>$spousename</a> ";
					#The three chart-generation icons next to the person's name
					$icons = generateIconLinks($spouserow, 'Desc', "$trail;{$spouserow['personID']}");
					$spousenamestr .= $icons;
					$alreadyprinted += substr_count($icons,'<img')*2.3;
					if ($srights['both']) {
						$spousedatastr = getVitalDates( $spouserow );
						#Show marriage date only if the marriage is (i.e. both spouses are) defined as non-living, non-private
						if ($mrights['both'] && $rrconfig['Text+ShowMarriage']) {
							if ($rrconfig['Text+Date'])
								$marriagestr = rrGetDisplayDate($row['marrdate'], $row['marrdatetr']);
							if (strcasecmp($marriagestr,$text['tp-unknownvalue'])==0) $marriagestr="";
							if ($rrconfig['Text+Place'] && $row['marrplace'])
								$marriagestr .= ($marriagestr ? " " : "") . $row['marrplace'];
							if ($marriagestr) {
								$marriagestr = "{$text['tp-married1']}. $marriagestr";
								if (!$rrconfig['Text+3Lines']) $marriagestr = "($marriagestr)";
							}
						} else $marriagestr = '';
					} else $spousedatastr = '';
				} #End if there is a person record for this spouse
			} #end if is there a spouse.
			//Start a div for this spouse, with indentation. The indent function updates $spaceleft
			$rval .= indent($spaceleft, $level);
			//Set $sbars so that we can connect multiple marriages to each other
			$sbars[$level]=($marriagecount < $numberofmarriages) ? 1 : 0;

			#If we printed generation numbers for the descendant, then space over to keep the names lined up.
			if ($generationnumbers) {
				$rval .= str_repeat( '&nbsp;', strlen($generationnumbers));
				$spaceleft -= strlen($generationnumbers);
			}

			#Don't bother counting children if we know we are not going to print any children.
			$doAnotherGeneration = false;
			$vabars[$level]=0;
			$plusminus = '+&thinsp;'; #default = the actionless plus sign next to a spouse without children
			if (!$params['hidedescend'] || ($prightsboth && $srights['both'])) {
				$result2 = getChildrenData($tree, $row['familyID']);
				$numkids = tng_num_rows( $result2 );
				#Allow expansion of the plus sign next to the spouse if there are children and available
				#chart generations, unless both parents are unrestricted or we're hiding descendants
				#of living people).
				if( $numkids && $level <= $params['genstodo']) {
					$doAnotherGeneration = true;
					$vbars[$level]=1;
					$divname = "fc$divctr";
					$divctr++;
					$plusminus = "&thinsp;<img class='plusminus fakelink' id='plusminus$divname' src='{$cms['tngpath']}" . "img/$excolimg.gif' width='9' height='9' hspace='0' vspace='0' border='0' title='$imgtitle' alt='' />&thinsp;";
				}
			} else $result2 = '';
			#Print the spouse before acting on the doAnotherGeneration flag that was just set.
			$rval .= $plusminus . $spousenamestr;
			$rval .= wrapData( $spaceleft, $level, strlen($spousename), $spousedatastr, $marriagestr, "");

			#Now act on the doAnotherGeneration flag
			if ($doAnotherGeneration) {
				#Start the block for the spouse's children
				$rval .= "<div id='$divname' style='display:$display;'>";

				if (isset($families[$row['familyID']])) {
					//We've seen this family before
					$vbars[$level] = 2;
					#The indent function updates $spaceleft
					$rval .= indent($spaceleft, $level, "person") . " ";
					$vbars[$level] = 0;
					$datastr = $text['tp-werecousins'];
					$rval .= wrapData( $spaceleft, $level, 0, $datastr, "", "", "person");
				} else {
					$families[$row['familyID']] = ""; //Save this family in the list of familes seen.
					//Display the children
					$countkids = 0;
					while( $crow = tng_fetch_assoc( $result2 ) ) {
						$countkids++;
						$newtrail = "$trail,{$row['familyID']},{$crow['personID']}";
						$crights = determineLivingPrivateRights($crow, $righttree);
						$crow['allow_living'] = $crights['living'];
						$crow['allow_private'] = $crights['private'];
						$cname = getName( $crow );
						$vbars[$level] = $countkids<$numkids ? 3 : 2;
						#The indent function updates $spaceleft
						$rval .= indent($spaceleft, $level, "person") . " ";
						$vbars[$level] = $vbars[$level]-2;
						$datastr = "";
						$continuelink = ""; $continuelinklen = 0;
						#Don't hyperlink the name in pretty print mode.
						if ($tngprint)
							$namelink = $cname;
						else $namelink = "<a href='$getperson_url" . "personID={$crow['personID']}&amp;tree=$tree'>$cname</a>";
						$continuelink = $datastr = '';
						$continutelinklen = 0;
						if ($crights['both'])
							$datastr = getVitalDates( $crow );
						if( $level == $params['genstodo'] && (!$params['hidedescend'] || $crights['both'])) {
							if(checkForChildren($tree,$crow['personID'])) {
								$continuelink = "&nbsp;&nbsp;<a href='{$textplus_url}personID={$crow['personID']}&amp;tree=$tree' class='tpleaflink' title=\"{$text['popupnote3']}\">=></a>";
								$continuelinklen = 2;
								$count['leaflinks']++;
							}
						}
						if ($params['showgennums']) {
							$gennums = PrintGenerationNumbers2($level,$countkids, $params['showsiblingsums']);
							$rval .= "<span style='font-size:90%'>&thinsp;$gennums&thinsp;</span>";
							$alreadyprinted += strlen($gennusm);
						}
						$rval .= $namelink;
						$alreadyprinted = strlen($cname);
						$count["descendants"]++;
						#The three chart-generation icons next to the person's name
						$icons = generateIconLinks($crow, 'Desc', $newtrail);
						$rval .= $icons;
						$alreadyprinted += substr_count($icons,'<img')*2.3;
						# Print the person's data
						$rval .= wrapData( $spaceleft, $level, $alreadyprinted, $datastr, "", $continuelink, "person");

						#Continue to the next generation unless the person is living or private.
						$ind = getIndividual( $crow['personID'], $crow['sex'], $level + 1, $newtrail, (!$params['hidedescend'] || $crights['both']), $gennums);
						if($ind) $rval .= "$ind\n";
					} #End of loop through children
				} #End of if we have seen this family before
				#End the div for this spouse's descendants
				$rval .= "</div>\n"; # end $divname -->\n";
			}
			if ($result2) tng_free_result( $result2 );
		}
	}
	tng_free_result( $result );
	return $rval;
} # end of function getIndividual

######################################################################
/** checkForChildren just counts a person's children (by all spouse)
* It uses the 'union' form of the getSpouseFamilyMinimum function because it
* doesn't know whether the person is male or female.
*/
function checkForChildren($tree,$personID) {
    $nxtfams = getSpouseFamilyMinimalUnion($tree, $personID);
    $nxtkids = 0;
    while(($nxtfam = tng_fetch_assoc($nxtfams)) && $nxtkids==0) {
        $result3 = countChildren($tree, $nxtfam['familyID']);
        $nxtrow = tng_fetch_assoc($result3);
        $nxtkids += $nxtrow['ccount'];
        tng_free_result($result3);
    }
    return $nxtkids;
} #end function checkForChildren

######################################################################
/** function indent returns the monospaced text that indents each line of the chart.
*   $charactersleft is passed by reference, and is only an output parameter.
*       It will be the number of characters that can be printed on this line after the after the indentation.
*   $level is the generation number, which, of course controls how many levels of
*       indentation need to be generated.
*   $what can contain "person" and/or "spouse" and/or "wrap".
*       If this line is wrapped from a previous line, we have to generate the line break before
*       the indentation, and, if not, then we start the line with a div whose classes indicate
*       whether this is a descendant or a spouse.
*/
function indent (&$charactersleft, $level, $what = "") {
	global $params, $vbars, $draw, $sbars, $drawvbar, $count, $text;
	$count["linesprinted"]++;
	$indenttext = "";
	$wraptext = "";
	for ($i=2; $i<=$level-1; $i++) {
		$sbar = $sbars[$i] ? $draw["spousebarOn"] : $draw["spousebarOff"];
		$indenttext .= $sbar . str_repeat( "&nbsp", $params['indentation']-3) . $drawvbar[$vbars[$i]] . $draw['hspaces'];
	}
	//Calculate inches used by the fixed-sized indentation. $fontsize is in points, and there are 120 pts/in
	$charactersused = $params['indentation']*($level-2);
	if (strpos ($what,"person") !== false) {
		#This is a descendant, not a spouse.
		$sbar = $sbars[$level] ? $draw["spousebarOn"] : $draw["spousebarOff"];
		$indenttext .= $sbar . str_repeat( "&nbsp", $params['indentation']-3) . $drawvbar[$vbars[$level]];
		$charactersused += $params['indentation']; //Counts the connector below
		if (strpos ($what, "wrap") === false) {
			$indenttext .= $draw["hbar"];
			$wraptext = "<div class='ppl'>"; #This line will be a person who is a descendant
		} else {
			$indenttext .= $draw["hspaces"]. "&nbsp;";
			$charactersused += 1; #1 character for wrapped line indentation
			$wraptext = "<br/>";
		}
	} else { #Spouse
		if (strpos ($what,"wrap") !== false) {
			$indenttext .= ($sbars[$level] ? $draw["spousebarOn"] : $draw["spousebarOff"]) . $draw['hspaces'];
			$charactersused += 2;
			$wraptext = "<br />";
		} else $wraptext = "<div class='ppl sp'>"; #This line will be a person who is a spouse
	}

	//To calculate the number of characters left, we find the inches used by the fixed font
	//indentation, and then determine the number of characters of proportional font that is
	//available, and then add the "widthplus" fudge factor.
	$inchesused = $charactersused/$params['fixedCPI'];
	$charactersleft = floor(($params['pagewidth']-$inchesused)*$params['proportionalCPI'] + $params['widthplus']);
	#If there is room for fewer than 10 characters, then print an error message
	if ($charactersleft<10) {
		$charactersleft=999;
		return "$wraptext<span style='font-weight:bold;color:red'>{$text['tp-nofit']} $level: </span>\n";
	} else
		return "$wraptext<span class='tpindents'>$indenttext</span>";
} #end function indent

######################################################################
/** Given two strings (e.g. a name and the person's dates) that need to be printed,
*   function wrapData returns the content of those strings broken into multiple lines of text,
*   as necessary. That is, if we need to wrap the line, the returned string will contain the
*   required number of instances of a <br> tag and indentation text.
*   Input arguments:
*   $spaceleft is the number of printable character positions that follow the indentation. Some
*         of them may be used (see $alreadydisplayedlength)
*   $level is the number of generations (i.e. the indentation leve), which we need if we
*         have to wrap the text to a new line.
*   $alreadydisplayedlength tells us how much of $spaceleft has already been used.
*   $datastr & $secondstr are the two strings that need to be printed.  If they don't both fit
*         on the line, our preference is to break between them rather than break within them.
*   $continuelink may be empty, or it may be the hyperlined symbol that does at the end of
*         a line to indicate that this person has addition children who are not in this chart.
*         Note that, when there is wrapping, $continuelink is printed on the first line, with
*         the person's name, not on the last wrapped line.
*   $what should already contain 'person' or 'spouse', and we'll add 'wrap' to as we call
*         the indent function to wrap the line.
*/
function wrapData( $spaceleft, $level, $alreadydisplayedlength, $datastr, $secondstr, $continuelink, $what="") {
	global $rrconfig;

	$debug=false;
	$wrappedlinespaceleft = $spaceleft - (strpos($what, "wrap")===false ? 1 : 0);
	$spaceleft = $spaceleft - $alreadydisplayedlength;
	$strlen1 = strlen($datastr);
	$strlen2 = strlen($secondstr);
	$newtext = "";

	#If just printing the name has already exceeded the space available, start a new line.
	#The indent function updates $spaceleft
	if ($spaceleft<=9 && $strlen1+$strlen2)
		$newtext = indent($spaceleft, $level, $what."wrap");

	if ($rrconfig['Text+3Lines'] && ($datastr || $secondstr)) {
		$datalines = explode('||', $datastr);
		$newtext .= $datalines[0];
		if (isset($datalines[1])) $newtext .= indent($spaceleft, $level, $what.'wrap') . '&nbsp;'.$datalines[1];
		if (isset($datalines[2])) $newtext .= indent($spaceleft, $level, $what.'wrap') . '&nbsp;'.$datalines[2];
		if ($secondstr) $newtext .= indent($spaceleft, $level, $what.'wrap') . '&nbsp;'.$secondstr;
		$newtext .= $continuelink;
	}
	elseif ($spaceleft >= ($strlen1 + $strlen2)) {
		#They both fit on this line
		$newtext .= $datastr . $secondstr . $continuelink;
if ($debug) $newtext.="[1 $spaceleft ($alreadydisplayedlength) $strlen1 $strlen2]";
		$datastr = $secondstr = "";
	}
	elseif ($spaceleft >= $strlen1 && $wrappedlinespaceleft >= $strlen2) {
		#They fit nicely on this and a second line. The indent function updates $spaceleft
		$newtext .= "$datastr\n" . indent($spaceleft, $level, $what."wrap") . $secondstr . $continuelink;
		$datastr = $secondstr = "";
	}
	else {
		//Join the two strings and break them as necessary. Note that breakLine lops off the
		//return value from the beginning of $mystring, and modifies $spaceleft.
		$mystring = $datastr . $secondstr;
		while ( $spaceleft < strlen($mystring) ) {
			$xx = $debug ? ("[3 $spaceleft $strlen1 $strlen2 " . strlen($mystring) . "]") : "" ;
			$newtext .= breakLine($mystring, $spaceleft, $wrappedlinespaceleft) . $xx. indent($spaceleft, $level, $what.'wrap');
		}
		$newtext .= $mystring . $continuelink;
	}
	return $newtext . "</div>\n";
} #end function wrapData

?>