<?php
##### rrtextplus_descend_tracker_ajx.php, installed by ###TextPlus Charts, by Robin Richmond
/* This is an AJAX program that is called from the TextPlus Descendant chart to
generate a popup div that shows where a person fits in the chart.
It replaces, for this purpose, (but can invoke) the old desctracker.php program
that displays more information but takes the user away from the context of the chart.
Expected querystring parameters:
 $tree,
 $trail - comma-delimited list": startingpersonID,familyID,descendantID,familyID,descendantID...
          where the familyIDs represent the person's marriage, and the final descendant is not
		  followed by a marriage but could be followed by ";spouse".
*/
$tngModNames['TextPlus Charts']['rrtextplus_descend_tracker_ajx.php'] = '17'; #Register this mod
#######################################################################

$textpart = "pedigree";
include("tng_begin.php");
include($subroot . "pedconfig.php");
#Read custom language strings from a mods subfolder rather than from regular cust_text.php files.
$modfolder = $text['tp-modfolder'];
if (file_exists("$modspath/$modfolder/{$mylanguage}_cust_text.php"))
	include("$modspath/$modfolder/{$mylanguage}_cust_text.php");
else include("$modspath/$modfolder/languages/English_cust_text.php");

$righttree = checktree($tree);

#The trail contains a personID and familyID for each descendant and spouse, except for the
#last descendant, for which there is no familyID.
$items = explode( ',', $trail );

#Approximate the box width (in px) based on the number of generations, and derive the height
#from the number of people displayed.
$width = 350 + 28*count($items);
$height = 66; #Take into account the banner, the link to the larger chart, and a possible blank line.

$indenteach = 4; #Number of non-breaking spaces by which each generation is indented.
$indentnegative = 4; #Each person's data line uses negative indentation.
$indenttotal = $indentnegative; #Start with indentation that accommodates the negative indent
#Indentation for the spouse replaces the line drawn for the descendant, plus the generation number.
$spousespaces = str_repeat('&nbsp;',6);
$leader1 = '&#9492;&#9472&#9472;'; #Line connection to the descendant
$leader2 = str_repeat('&nbsp;',3); #The descendant in generation 0 gets spaces instead.

# Loop through the trail, counting two elements (personID and familyID) for each generation.
# We'll get two people records, and make sure that the descendant is in $spouse[0],
# and the spouse is in $spouse[1]
$html = '';
for ($g=0; $g<count($items); $g+=2) {
	$spouses =  array();

	if ($g < count($items)-1){
		#Get person and spouse via the familyID. The query will result in up to two person records.
		#The spouse could be null, in which case there will be only one record.
		#When there are two records, the husband comes first, and the wife comes second.
		$query = "SELECT personID, lastname, lnprefix, firstname, prefix, suffix, nameorder,
			birthdate, altbirthdate, deathdate, burialdate, p.living, p.private, p.branch, sex
		FROM $families_table f
		LEFT JOIN $people_table p ON (p.personID=f.wife OR p.personID=f.husband) AND p.gedcom=f.gedcom
		WHERE f.familyID='{$items[$g+1]}' and f.gedcom='$tree'";
		$result = tng_query($query);
		$spouses[0] = tng_fetch_assoc($result);
		#If there are two records in the result set, get the second record (the spouse),
		#and possibly switch the spouses to make sure that the descendant comes first.
		#We don't care about the sex of each spouse from here on.
		if (tng_num_rows($result)>1) {
			$spouses[1]  = tng_fetch_assoc($result);
			if ($items[$g] != $spouses[0]['personID']) {
				$x = $spouses[0]; $spouses[0]=$spouses[1]; $spouses[1]=$x;
			}
		}
	} else { #This is the leaf of the descendancy; just get date for this one person.
		#The final generation could be (descendant;spouse)
		$pair = explode(';', $items[$g]);
		for ($s=0; $s<count($pair); $s++) {
			$query = "SELECT personID, lastname, lnprefix, firstname, prefix, suffix, nameorder,
				birthdate, altbirthdate, deathdate, burialdate, living, private, branch, sex
			FROM $people_table
			WHERE personID='{$pair[$s]}' and gedcom='$tree'";
			$result = tng_query($query);
			$spouses[$s] = tng_fetch_assoc($result);
		}
	}
	for ($s=0; $s<count($spouses); $s++) {
		#Generate the indentation for this generation
		$html .= "<div style='margin-left:{$indenttotal}em; text-indent:-{$indentnegative}em;'>";
		$spousenamestr = $spousedatastr = '';
		if( $spouses[$s]['personID'] ) {
			$height +=16;
			if ($s==0) { #The descendant. Always include the generation number.
				$html .= "<span class='tpindents'>";
				if ($g==0) $html .= "$leader2</span>";
				else $html .= "$leader1</span>";
				$html .= '&nbsp;' . ($g/2+1) . '. ';
			} else #the spouse
			$html .= "<span class='tpindents'>$spousespaces</span>{$text['tp-married1']}. ";

			$prights = determineLivingPrivateRights($spouses[$s], $righttree);
			$spouses[$s]['allow_living'] = $prights['living'];
			$spouses[$s]['allow_private'] = $prights['private'];
			$html .= "<span>" . getName($spouses[$s]) . "</span>";

			#Always save this person's data row. The last person  we encounter is the "leaf"
			#descendant, who goes into the title bar and chart links
			$thispersonrow = $spouses[$s];

			#We can get the birth and death dates only if we have rights.
			#When a date is missing, look for the alternate date.
			if ($prights['both']) {
				$dob = $spouses[$s]['birthdate'];
				if (!$dob || !strcasecmp($dob, 'unknown') || !strcasecmp($dob, $text['tp-unknownvalue'])) {
					#Get alternate
					$dob = $spouses[$s]['altbirthdate'];;
					if (!$dob || !strcasecmp($dob, 'unknown') || !strcasecmp($dob, $text['tp-unknownvalue']))
						$dob = $text['tp-unknowndisplay']; #Unknown
					else $dob = "{$text['tp-chrabbr']}$dob"; #Alternate
				}
				#Death date is superseded by the Living flag.
				if ($spouses[$s]['living']) $dod = $text['living'];
				else {
					$dod = $spouses[$s]['deathdate'];
					if ($dod == 'Y') $dod=''; #Y is no better than nothing at this point
					if (!$dod || !strcasecmp($dod, 'unknown') || !strcasecmp($dod, $text['tp-unknownvalue'])) {
						#Get alternate
						$dod = $spouses[$s]['burialdate'];
						if (!$dod || !strcasecmp($dod, 'unknown') || !strcasecmp($dod, $text['tp-unknownvalue']))
							$dod = $text['tp-unknowndisplay']; #Unknown
						else $dod = "{$text['tp-chrabbr']}$dod"; #Alternate
					}
				}
				$mydates = "($dob-$dod)";
				if ($mydates=='(-)' || $mydates=="({$text["tp-unknowndisplay"]}-{$text["tp-unknowndisplay"]})")
					$mydates = '';
				elseif ($mydates == "({$text["tp-unknowndisplay"]}-{$text['living']})")
					$mydates = "({$text['living']})";
			} else $mydates = '';
			$html .= " $mydates</div>\n";
		} # end of if there is a spouse record
	} #end for each spouse
	$indenttotal += $indenteach;
} #end for each generation

#The person of interest is the last person seen
$thispersonname = getNameUniversal($thispersonrow, 1);
$thispersonID = $thispersonrow['personID'];

# Chart links
$html .= "<div><a href='desctracker.php?tree=$tree&trail=$trail'>{$text['tp-graphicalview']}</a>
	&ensp;&ensp;||&ensp;&ensp;
	<a href='rrtextplus_descend.php?personID=$thispersonID&amp;tree=$tree'
		title='{$text['descendants']}'>{$text['tp-printdesc']}</a>
	&ensp;&ensp;||&ensp;&ensp;
	<a href='rrtextplus_ancestor.php?personID=$thispersonID&amp;tree=$tree'
		title='{$text['ancestors']}'>{$text['tp-printanc']}</a>
	</div>\n";

#Place the chart in a box that will be popped-up by the calling program.
#Generate the box in which the text will be placed
echo "<div id='trackdiv2'>
<div id='trackbanner'>
	<div class='rrtpBoxButton'><img src='img/tng_close.gif' class='rrtpCloseX'></div>
	<div class='rrtpBoxBanner'>{$text['tp-locate']} $thispersonname</div>
</div>
<div id='tracktext' class='rrtpBoxText'>$html</div>
</div>\n";
?>
