<?php
##### rrtextplus_lib.php, installed  by ###TextPlus Mods, by Robin Richmond
# This is a Javascript & PHP function library file for use by the TextPlus Charts programs.
# Text-formatting functions, and a function to translate name & date query-string
# parameters to the person ID that the programs normally expect.
#
/****** History ******
v16 25 Jun 2019
- Cleaned up technical errors such as unitialized variables that were flagged by PHPv7.2.
v15e 21 Nov 2018
- The initial page widths are derived from TNG system variables rather than mod parameters
- Cleaned up several PHP warnings related to uninitialized variables.
- Moved the Highlight Generation functions to rrtextplus_ancestor.php because it only exists there.
- The Clear Cookie button is not displayed if there is no cookie to clear.
- Fixed the broken code to clear the cookie.
v15b 31 Oct 2018
- Changed the +/- fudge factor input field on the Format for Printing form to type='numeric'
- Added multi-byte support to the wrapping functions - for non-English characters
v15 - 26 Oct 2018
- Changed the names of the two mod parameters used in the initializeParameters function.
- Changed the filename to rrtextplus_lib.php
- Changed the Format for Printing "Save Settings" button to a checkbox.
- Changed the Format for Printing submit button to call setPrintFormatCookie (which used to be
  called from the "Save Settings" button.
- The Save Settings cookies no longer save the Generations parameter (which was never used), so
  the array indexes for the cookie fields have been adjusted.
v14 - 14 Mar 2018
- Renamed from textpluslib.php to rrtextpluslib.php
- Now removes "XXONCLICKXX" from the chart content controls in a common menu.
- Redefined custom style classes to start with 'tp' rather than 'cg'.
- Added "mobile" to the paper sizes
- Reorganized what was the "Chart Formatting Form" on the Print for Formatting page to separate the
  Chart Formatting options and the Chart Content options into two separate fieldsets.
- On the Format for Printer page, moved the generation button to the Chart Content fieldset.
- New function to generate the Focus on Generation control
v13 21 Oct 2017
- Genericized the initializeParameters function and moved it here from descendtextplus and pedigreetextplus.php
v12 11 May 2016
- The custom variable $useCustomDatabase has been changed to $useShortPlacenames.
- Removed the database functions, which were legacies of a private mod.
v11 - 20 Nov 2015
- Registers the mod and using the Display Mod Names conventions.
v10b 7 Sep 2015
- in getPersonID, return the count of matching records as well as the personID and tree of the first match.
v10 25 May 2015
- Added function getPersonID to lookup a ID given name and optional birthyear and tree.
v10 25 Feb 2015
- For TNG v10.1
v9 5 Apr 2015
- In getVitalDates, check for alternate birth and death when birth or death is null.
*/
$tngModNames['TextPlus Charts']['rrtextpluslib.php'] = '16c'; #Register this mod

######################################################################
function getSpouseFamilyPrintData($tree, $spouse1, $spouse1ID, $spouseorder) {
#Get most family data for a known spouse, with alternate place names for marriage and divorce
	global $families_table, $places_table, $tngconfig;

	#There are two forms of the query, in anticipation of the possibility that the Short Place Names query might be installed.
	if ($GLOBALS['useShortPlacenames']) {
		#If all places are stored in one tree, we don't want to use the tree ID when joining to Places.
		if ($tngconfig['places1tree'])
			$placejoin1 = $placejoin2 = ''; #Don't need to join on Gedcom
		else { #We need to join on Gedcom
			$placejoin1 = ' AND p1.gedcom = f.gedcom';
			$placejoin2 = ' AND p2.gedcom = f.gedcom';
		}
		#To get the short placenames, join to two instances of the Places table,
		#using the place ID foreign keys for marriage and divorce. forei
		$query = "SELECT f.gedcom, husband, wife, familyID, marrdate, marrtype, divdate, living, private, branch,
			IF (p1.shortplace IS NOT NULL, p1.shortplace, marrplace) AS marrplace,
			IF (p2.shortplace IS NOT NULL, p2.shortplace, divplace) AS divplace
			FROM $families_table f
			LEFT JOIN $places_table p1 ON p1.ID = f.pomarr $placejoin1
			LEFT JOIN $places_table p2 ON p2.ID = f.podiv $placejoin2
			WHERE $spouse1 = '$spouse1ID' AND f.gedcom = '$tree'
			ORDER BY $spouseorder";
	} else { #Just use the marriage and divorce places stored in the families table.
		$query = "SELECT f.gedcom, husband, wife, familyID, marrdate, marrplace, marrtype, divdate, divplace, living, private, branch
			FROM $families_table f
			WHERE $spouse1 = '$spouse1ID' AND f.gedcom = '$tree'
			ORDER BY $spouseorder";
	}
	return tng_query($query);
}

######################################################################
#Retrieve the result set for the most recent residence or census event.
function getPersonLastResidence($tree, $personID) {
	global $events_table, $eventtypes_table, $places_table, $tngconfig;

	#There are two forms of the query, in anticipation of the possibility that the Short Place Names query might be installed.
	if ($GLOBALS['rrconfig']['ShortPlacenames'][$tree]) {
		#If all places are stored in one tree, we don't want to use the tree ID when joining to Places.
		$placejoin = ($tngconfig['places1tree']) ? '' : " AND p1.gedcom = e.gedcom";
		#Use the event place to join to the Places table to get the "shorter" placename.
		$query = "SELECT eventdate, eventdatetr, if(shorterplace IS NOT NULL, shorterplace, shortplace) AS eventplace
			FROM $events_table e
			INNER JOIN $eventtypes_table et ON et.eventtypeID=e.eventtypeID
			LEFT JOIN $places_table p1 ON p1.id = e.poe $placejoin
			WHERE persfamID = '$personID' AND e.gedcom = '$tree'
				AND (et.tag='RESI' OR et.tag='CENS')
				AND keep=1 AND parenttag=''
			ORDER BY eventdatetr DESC limit 1";
	} else { #Just get the event place from the events table.
		$query = "SELECT eventdate, eventdatetr, eventplace
			FROM ($events_table e, $eventtypes_table et)
			WHERE persfamID = '$personID' AND e.gedcom = '$tree'
				AND e.eventtypeID = et.eventtypeID
				AND (et.tag='RESI' OR et.tag='CENS')
				AND keep = 1 AND parenttag = ''
			ORDER BY eventdatetr DESC LIMIT 1";
	}
	$result = tng_query($query);
	return $result;
}

######################################################################
# Given a data record, return a data string in one of these forms (always with parens, unless empty):
# (dob pob - dod pod), (place of birth and death; dob - dod), (dob pob - "Living" dor por), or blank.
# Missing places are always blank. Missing dates can be blank or "??", depending on what other data exists.
function getVitalDates($dr) {
if (count($dr)<30) var_dump($dr);
	global $session_charset; #Needed for multibyte character handling
	global $rrconfig; #Configuration parameters that control the format of the date/place string

	#Don't add data to living or private people if the user doesn't have permission to do so
	if( !$dr['allow_living'] || !$dr['allow_private'] )
		return "";

	global $text;        #array of multi-language strings
	global $yearsAgo21;  #yyyy-mm-dd exactly 21 years ago. (We don't show dor and por for people under 21)
	global $tree;        #The querystring parameter that is the tree of the person who started the report.
	$strdate = "";
	$strPlace = "";
	$dates = ""; #the combined birth/death date/place string

	#Note that DOB="" means "Unknown", but DOD="" does not. It may be associated with "Living".
	$usedAltBirthDate = false;
	$dob = $dod = '';
	if ($rrconfig['Text+Date']) { #if we're supposed to show dates
		#Make sure that "unknown" is specified in the user's language.
		if (!strcasecmp($dr["birthdate"], 'unknown'))
			$dob = $text['tp-unknownvalue'];
		else $dob = rrGetDisplayDate($dr['birthdate'], $dr['birthdatetr']);
		if (!strcasecmp($dob, $text['tp-unknownvalue']) ) $dob="";
		#When birth is empty, try the alternate birthdate
		if (!$dob) {
			if (!strcasecmp($dr["altbirthdate"], 'unknown'))
				$dr["altbirthdate"] = $text['tp-unknownvalue'];
			if ($dr['altbirthdate'] && strcasecmp ($dr['altbirthdate'],$text['tp-unknownvalue']) ) {
				$dob = rrGetDisplayDate($dr['altbirthdate'], $dr['altbirthdatetr']);
				$dob = $text['tp-chrabbr'] . $dob;
				$usedAltBirthDate = true;
			}
		}
		#Now DOD. If death is unknown, try burial date.
		$usedBurialDate = false;
		if (!strcasecmp($dr["deathdate"], 'unknown'))
			$dr["deathdate"] = $text['tp-unknownvalue'];
		$dod = rrGetDisplayDate($dr['deathdate'], $dr['deathdatetr']);
		if ( !$dod || !strcasecmp ($dod,$text['tp-unknownvalue']) ) {
			#Null or unknown date of death; try burial
			if (!strcasecmp($dr["burialdate"], 'unknown'))
				$dr["burialdate"] = $text['tp-unknownvalue'];
			if ($dr['burialdate'] && strcasecmp ($dr['burialdate'],$text['tp-unknownvalue']) ) {
				$dod = $text['burialabbr'] . rrGetDisplayDate($dr['burialdate'], $dr['burialdatetr']);
				$usedBurialDate = true;
			} else $dod= $text["tp-unknowndisplay"]; #Still unknown; make is "??"
		}
	} #End if we want a date at all.

	$pob = $pod = '';
	if ($rrconfig['Text+Place']) { #if we're supposed to showplaces
		#Get the place of birth or alternate place of birth,
		$pob = $dr['birthplace'];
		if ( !strcasecmp ($pob,"unknown") || !strcasecmp ($pob, $text['tp-unknownvalue']) || $pob=='??')
			$pob = ""; # Change "unknown" to ""
		if ($pob) {
			#When we use the alternate birthdate, but have a birthplace, we need to mark the birthplace with "b."
			if ($usedAltBirthDate) $pob = $text['birthabbr'] . $pob;
		} else {
			$p = $dr['altbirthplace'];
			if ( !strcasecmp ($p,"unknown") || !strcasecmp ($p, $text['tp-unknownvalue']) || $p=='??')
				$p = ""; # Change "unknown" to ""
			if ($p) {
				#We want to use the alternate birth date
				if ( !$usedAltBirthDate ) $pob =  $text['tp-chrabbr'];
				$pob .= $p;
			}
		}

		#Now death or burial place
		$pod = $dr['deathplace'];
		if ( !strcasecmp ($pod,"unknown") || !strcasecmp ($pod, $text['tp-unknownvalue']) || $pod=='??')
			$pod = ""; # Change "unknown" to ""
		#If no Place of Death, use the Burial place.
		if (!$pod) {
			$p = $dr['burialplace'];
			if ( !strcasecmp ($p,"unknown") || !strcasecmp ($p, $text['tp-unknownvalue']) || $p=='??')
				$p = ""; # Change "unknown" to ""
			if ($p) {
				#We want to use the burial place
				$pod = "";
				if ( !$usedBurialDate ) $pod =  $text['burialabbr'];
				#If "Cemetery," is in the placename, assume that it is at the end of a cemetery name, and
				#remove the full cemetery name.
				#Make sure the "Cemetery," is expressed in the user's language.
				if (!strcasecmp($p, "Cemetery,"))
					$p = str_ireplace('Cemetery,', "{$text('cemetery')},", $p);
				if ($session_charset == 'UTF-8') {
					if (($c = mb_strpos($p, "{$text['cemetery']},", 0, $session_charset)) !== false)
						$p = mb_substr($p, $c+mb_strlen($text['cemetery'])+1, mb_strlen($p, $session_charset), $session_charset);
				} else {
					if (($c = strpos($p, "{$text['cemetery']},")) !== false)
						$p = substr($p, $c+strlen($text['cemetery'])+1);
				}
				#If the placename parts are separated by comma-space, a leading space will remain
				#after removing a cemetery . Remove it, and any other leading or trailing spaces.
				$p = trim($p);
				$pod .= $p;
			}
		} elseif ($usedBurialDate) {
			#We used the alternate deathdate, so we need to mark the death place
			$pod = $text['deathabbr'] . $pod;
		}
	} #end if we want a placename at all

	#When there's no death date value, but there is a place of death, then we can
	#declare that we have an unknown death date.
	if (!$dod && $pod)  $dod = $text['tp-unknowndisplay'];

	#When alive, then use date and place of last residence
	#(Hmm...I wonder if this test should only be applied if we have not found a death date.)
	if ($dr["living"]) {
		#The person is alive
		#Omit date and place of residence for (living) people born fewer than than 21 years ago.
		#We assume they are living with their parents or maybe at college.
		#It would be better to remove the place of residence only if the child has the same
		#residence as the parents. But we have no way of knowing that. Thus, this scheme could
		#remove a useful piece of information.
		if ($dr["birthdatetr"] > $yearsAgo21) {
			$dod = $pod = '';
		} else {  #The living person is over 21; try to show the last residence date.
			$dod = $text['living']; #The date of death starts with "Living"
			$result = getPersonLastResidence($tree, $dr['personID']);
			if ($result) { #There's a residence event that we can display.
				$residenceRow = tng_fetch_assoc( $result );
				if ($rrconfig['Text+Place']) $pod = $residenceRow['eventplace'];
				#If there's a residence date, then include it in the date.
				if ($residenceRow["eventdate"])
					$dod .= ' ' . rrGetDisplayDate($residenceRow["eventdate"], $residenceRow["eventdatetr"]);
			}
		}
	}

	#Now format the date/place data, if there is useful data to display.
	if (($dob == "" || $dob == $text['tp-unknowndisplay'])
		&& ($dod == "" || $dod == $text['tp-unknowndisplay']
		&& !$pob && !$pod)) {
		$dates  = ""; #There's no meaningful data to display
	} else {  #There is data. Format it differently if the two places are the same.
		if ($rrconfig['Text+3Lines']) {
		$dates = '';
		if ($dob || $pob) $dates .= "||b. $dob $pob";
			if ($dod || $pod) {
				$dates .="||";
				if (strpos($dod, $text['living']) !== 0)
					$dates .= "d. ";
				$dates .= "$dod $pod";
			}
		} else {
			if ($rrconfig['Text+SamePlace'] && $pob && $pod && $pob == $pod) {
				#The places are the same, so format it as (place;date - date)
				$dates = "$pob;$dob - $dod";
			} else {  #Different places of birth and death become (date place - date place)
				#Put birth date & place together
				if (!$pob || !$dob)
					$dates  = $dob . $pob; #one of them is blank; we don't need a separator
				else $dates = $dob . " " . $pob;
				#Add death date & place
				if (!$pod || !$dod)
					$dates .= " - " . $dod . $pod;
				else $dates .= " - " . $dod . " " . $pod;
			} #endif places of birth and death are the same
		} #end if 3lines #[[xyz]]
	}

	if ($rrconfig['Text+Age']) {
		$age = age($dr);
		if ($age) {
			#Remove space after a symbol that indicates relative or approximate age
			$age = preg_replace( '/([<>~]) /', '$1', $age);
			//if (substr($age, 2, 1)==' ') str_replace(substr($age, 0, 3), substr($age, 0, 2), $age);
			if ($rrconfig['Text+3Lines'])
				$dates = $age . $dates;
			else $dates = ($dates ? "$dates. " : '') . $age;
		}
	}
	if ($dates && !$rrconfig['Text+3Lines'])
		$dates  = "(" . $dates . ")"; #Put $dates in parens
	return $dates;
} #End function getVitalDates

###################################################################
#Use a mod option to determine the date format.
#The function displaydate is in globallib.php.
function rrGetDisplayDate ($mydate, $mydatetr) {
	global $rrconfig;
	if ($rrconfig['Text+Date']=='full')
		return displaydate($mydate);
	elseif ($rrconfig['Text+Date']=='year') {
		$newdate = substr($mydatetr, 0, 4);
		if ($newdate == '0000') return $mydate;
		else return $newdate;
	}
	else return '';
} #end function rrGetDisplayDate

######################################################################
function breakLine(&$mystring, $spaceleft, $nextlinespaceleft) {
#Lop off part of $mystring, returning the $prefix as the function value, and returning $mystring
#with just the remainder of $mystring. The return value cannot be longer than $spaceleft.
	global $session_charset; #needed for multibyte character handling

	if ($session_charset == 'UTF-8')
		$strlen1 = mb_strlen($mystring, $session_charset);
	else $strlen1 = strlen($mystring);
	#If the entire string will fit on this line, then just return it.
	if ($strlen1 < $spaceleft) {
		$prefix  = $mystring; #Return the entire string
		$mystring = "";
		return $prefix;
	}

	if ($strlen1 < $spaceleft + $nextlinespaceleft) {
		#The data should fit on this line and the next.

		#First, look for the semicolon that follows a place used by birth and death.
		#Then, look for the space-dash-space in (birth - death)
		#Use whichever fits on the line, and is longer.
		#(I'm assuming here that the string doesn't start with ';' or ' - '.)
		#(I'm also not sure why we I didn't look for the last ';' or ' - ' first.)
		if ($session_charset == 'UTF-8') {
			$c1 = mb_strpos($mystring, ";", 0, $session_charset);
			$c2 = mb_strpos($mystring, " - ", 0, $session_charset);
		}else { #ISO Character set
			$c1 = strpos($mystring, ";");
			$c2 = strpos($mystring, " - ");
		}
		if ($c1 > $spaceleft || $c1<9 || $strlen1-$c1<6) $c1=0;
		if ($c2 > $spaceleft || $c2<9 || $strlen1-$c2<6) $c2=0;

		#Use the breakpoint that came last
		if ($session_charset == 'UTF-8') {
			if ($c1 > $c2 ) {
				#Use the semicolon break
				if (mb_strlen($mystring, $session_charset)-$c1 > $nextlinespaceleft) {
					#The remainder won't fit on line 2, so don't use this break point
					$c1=0;
				} else {
					$prefix = mb_substr($mystring, 0, $c1+1, $session_charset);
					$mystring = mb_substr($mystring, $c1+1, mb_strlen($mystring, $session_charset), $session_charset);
					return $prefix;
				}
			}
			if ($c2 > $c1 && mb_strlen($mystring, $session_charset)-$c2 < $nextlinespaceleft) {
				#use the " - " break
				$prefix = mb_substr($mystring, 0, $c2+2, $session_charset);
				$mystring = mb_substr($mystring, $c2+2, mb_strlen($mystring, $session_charset), $session_charset);
				return $prefix;
			}
		} else { #ISO character set
			if ($c1 > $c2 ) {
				#Use the semicolon break
				if (strlen($mystring)-$c1 > $nextlinespaceleft) {
					#The remainder won't fit on line 2, so don't use this break point
					$c1=0;
				} else {
					$prefix = substr($mystring, 0, $c1+1);
					$mystring = substr($mystring, $c1+1);
					return $prefix;
				}
			}
			if ($c2 > $c1 && strlen($mystring)-$c2 < $nextlinespaceleft) {
				#use the " - " break
				$prefix = substr($mystring, 0, $c2+2);
				$mystring = substr($mystring, $c2+2);
				return $prefix;
			}
		}
	}
	#We don't have an easy way to fit on two lines, or we can't fit on two lines, so
	#just break the line where we can.
	#Don't allow orphans of less than 8 characters.  That is, if the string is no more than
	#8 characters longer than the space available, reduce the space available so that there
	#are at least 8 characters left over.
	if ($strlen1- 8 < $spaceleft)  $spaceleft = $strlen1 - 8;

	#Now, try to break on a space, a comma, or a dash, but not right before a close paren.
	if ($session_charset == 'UTF-8') {
		for ($c = $spaceleft; $c>8; $c--) {
			$char = mb_substr($mystring, $c ,1, $session_charset);
			$nextchar = mb_substr($mystring, $c+1, 1, $session_charset);
			if (($char == " "|| $char == "," || $char == "-") && $nextchar!=")") {
				$prefix  = mb_substr($mystring, 0, $c+1, $session_charset); #Keep the separator on line 1
				$mystring = mb_substr($mystring, $c+1, mb_strlen($mystring, $session_charset), $session_charset);
				return $prefix;
			}
		}
		#If that didn't work, just break the line at $spaceleft
		$prefix = mb_substr($mystring, 0, $spaceleft, $session_charset);
		$mystring = mb_substr($mystring, $spaceleft, mb_strlen($mystring, $session_charset), $session_charset);
		return $prefix;
	} else { #ISO character set
		for ($c = $spaceleft; $c>8; $c--) {
			$char = substr($mystring, $c ,1);
			$nextchar = substr($mystring, $c+1, 1);
			if (($char == " "|| $char == "," || $char == "-") && $nextchar!=")") {
				$prefix  = substr($mystring, 0, $c+1); #Keep the separator on line 1
				$mystring = substr($mystring, $c+1);
				return $prefix;
			}
		}
		#If that didn't work, just break the line at $spaceleft
		$prefix = substr($mystring, 0, $spaceleft);
		$mystring = substr($mystring, $spaceleft);
		return $prefix;
	} #end which character set
} #end function breakLine

######################################################################
function findhelp( $helpfile ) {
	#This function was copied directly from adminlib.php for use in non-admin programs.
	global $mylanguage, $language;

	if( file_exists("$mylanguage/$helpfile") )
		$helplang = $mylanguage;        #$mylanguage should already include "languages/"
	elseif( file_exists("languages/$language/$helpfile") )
		$helplang = "languages/$language";
	else
		$helplang = "languages/English";

	return $helplang;
} #end function findhelp

######################################################################
function debugdisplay($var,$linenumber="unknownline") {
	echo "<pre>at line $linenumber ";
	print_r($var);
	echo "</pre>";
} #end function debugdisplay

######################################################################
#Function getPersonID allows hyperlinks to specify a person's lastname (optionally) firstname prefix
#and/or birthyear so that they are not depending on possible changes in PersonID. (Of course, these
#alternate parameters are depending on name and birth year.)
#This function looks up a personID given the lastname, firstname, &/or birthyear
function getPersonID( $lname, $fname, $birthyear, $tree) {
	global $people_table;
	$returnvalue = array();

	if ($treerestrict && $assignedtree) {
			if ( $tree && $tree!=$assignedtree) return "";  #Not a legal tree
			if ( !$tree) $tree = $assignedtree; # Use the assigned tree since we're limited to it.
	}

	$query = "SELECT personID,gedcom FROM $people_table WHERE firstname LIKE \"%$fname%\" AND lastname LIKE \"%$lname%\"";
	if ( $birthyear ) $query .= " AND LPAD(SUBSTRING_INDEX(birthdate, ' ', -1),4,'0') = \"$birthyear\"";
	if ( $tree ) $query .= " AND gedcom=\"$tree\"";
	$result = tng_query($query);
	if( $result ) {
		$returnvalue['count'] = tng_num_rows($result); # The number of results
		$row = tng_fetch_assoc( $result ); # Get and return the first result.
		$returnvalue['personID'] = $row['personID'];
		$returnvalue['gedcom'] = $row['gedcom'];
	} else $returnvalue = "";
	tng_free_result($result);
	return $returnvalue;
} #end function getPersonID

######################################################################
#Generate an HTML <option> tag given the value (what gets passed), the default,
#and the text (what gets displayed)
function displaySelectionOption($value, $default, $text) {
	$selected = $value==$default ? ' selected' : '';
	return "<option value='$value'$selected>$text</option>";
} #end function displaySelectionOption

###################################################################
# Function to generate the control form in the Format for Printing page
#The form has two fieldsets - for Print Formatting controls and for Chart Content controls.
#The Text+ Descendant and Ancestor charts share four Print Formatting controls that are generated from
#scratch in this function.
# $commonmenu holds the controls that are in the second line of the "main" page innermenu, and that
# go in the Chart Content control fieldset in the Format for Printer page.
function generatePrintModeFormattingForm ($personID, $tree, $commonmenu) {
	global $text;
	global $params; #Formatting parameters - some are query string parameters, some not.

	$printmenu = "<div style='float:left;' class='tpmenu'>\n";
	$printmenu .= "<input type='hidden' name='personID' value='$personID'/>\n";
	$printmenu .= "<input type='hidden' name='tree' value='$tree'/>\n";
	$printmenu .= "<input type='hidden' name='tngprint' value='1'/>\n";

	#The first fieldset holds the Text+ printer page formatting fields.
	$printmenu .= "<fieldset style='padding:2px 3px 4px 5px'><legend style='font-weight:bold;'>{$text['tp-formatforprint']}</legend>\n";

	$printmenu .= "<label class='nw'>{$text['tp-indentation']}:<select name='indentation' id='indentation' class='verysmall'>";
	$printmenu .= displaySelectionOption(5, $params['indentation'] ,5);
	$printmenu .= displaySelectionOption(6, $params['indentation'], 6);
	$printmenu .= displaySelectionOption(8, $params['indentation'], 8);
	$printmenu .= displaySelectionOption(10, $params['indentation'], 10);
	$printmenu .= displaySelectionOption(12, $params['indentation'], 12);
	$printmenu .= displaySelectionOption(15, $params['indentation'], 15);
	$printmenu .= displaySelectionOption(18, $params['indentation'], 18);
	$printmenu .= "</select></label> &nbsp;\n";

	$printmenu .= "<label class='nw'>{$text['tp-fontsize']}:<select name='fontsize' id='fontsize' class='verysmall'>";
	for( $i = 7; $i <= 14; $i++ )
		$printmenu .= displaySelectionOption($i, $params['fontsize'], $i."pt");
	$printmenu .= "</select></label> &nbsp;\n";
	$printmenu .= str_replace('XXSUBMITXX', '', generatePageWidthField()) . " &nbsp;\n";

	$printmenu .="<label class='nw'>+/-<input type='number' name='widthplus' id='widthplus' 
		style='width:3.3em;' value='{$params['widthplus']}' />{$text['tp-characters']}</label> &nbsp;";
	$printmenu .="<label class='nw'>{$text['tp-height']}:<input type='number' name='height' id='height'
		min='0' step='10' value='{$params['height']}' style='width:3.8em;'>%</label> &nbsp;";

	#Save Print Formatting Cookie Button
	$printmenu .= "<div style='float:right;margin-left:1em;'><input type='button' id='savecookie' value='{$text['tp-savesettings']}'
		class='small' onclick=\"savePrintFormatCookie('{$params['phpBaseFilename']}');\"/>\n";
	#The Clear Cookie button is hidden if there is no cookie to clear.
	$cookiestyle = ($params['cookie']) ? '' : " style='display:none;'" ;
	$printmenu .= "&nbsp; <input type='button' id='clearcookie' class='small'$cookiestyle
		value='{$text['tp-clearcookie']}'
		onclick=\"clearPrintFormatCookie('{$params['phpBaseFilename']}');\"/>\n";
	$printmenu .= "</div></fieldset>\n";

	#The Options that are specific to the descendant or ancestor chart, and are shared
	#with those program's innermenus, are in a second fieldset for Chart Content controls.
	$printmenu .= "<fieldset style='padding:0px 3px 3px 3px;'><legend><b>{$text['tp-chartcontent']}</b></legend>
		$commonmenu</fieldset>\n";
	#close the left-floated div that wraps the fieldsets
	$printmenu .= "</div>\n";

	#Buttons - Submit & Help
	$helplang = findhelp("rrtextplus_help.php");
	$printmenu .=" <div class='tpprintbuttons'>
		<input type='submit' name='b1' value='{$text['tp-submit']}'>
		&nbsp; &nbsp; <input type='button' name='b2' value='{$text['help']}' onclick=\"return openTextPlusHelp('$helplang/rrtextplus_print_help.php');\"/>
		</div><div style='clear:left;'></div>\n";
	return $printmenu;
} #end function generatePrintModeFormattingForm

############################################################################
#Structural function that generates the HTML code for the Generations prompt.
function generateGenerationsSelectionBox($generationsallowed, $generationstodo) {
	global $text;
	$html = "<input type='hidden' id='gensallowed' name='gensallowed' value='$generationsallowed'>
	<label class='nw'>{$text['generations']}
		<select name='generations' id='generations' class='small' onchange='generationsChange()'>
		<option value='0'>{$text['all']}</option>
		</select>
	</label>\n";
	return $html;
} #end function generateGenerationsSelectionBox

###################################################################
# Generate the HTML for the Page Width selection box.
# This is done in a function because it appears both in the Format for Printing form and in the 
#"main" page Innermenu. It lists 4 report widths - 1 based on "Mobile", and 3 based on paper sizes.
function generatePageWidthField() {
	global $text, $params;

	$html = "<label class='nw'>{$text['tp-pagewidth']}:<select name='pagewidth' id='pagewidth' class='verysmall' onchange='selfSubmit();'>";
	$html .= displaySelectionOption(4, $params['pagewidth'], $text['tp-mobile']);
	$html .= displaySelectionOption(7.5, $params['pagewidth'], $text['tp-portrait']);
	$html .= displaySelectionOption(10, $params['pagewidth'], $text['tp-landscape']);
	$html .= displaySelectionOption(13, $params['pagewidth'], $text['tp-legal']);
	$html .= "</select></label>";
	return $html;
} #end function generatePageWidthField

######################################################################
# Initialize a set of global arrays and scalar variables. Some are based on the cookies or
#query-string parameters that which are passed in as global variables, some are effectively
#constants, and two are control arrays that control the printing of vertical lines.
# Input argument:
#    $whichReport = 'descend' or 'pedigree' (the program name prefix)
# The arrays to be created:
#    $params - Holds the parameters the report needs (which are, of course, derived from the
#              $query-string parameters, plus some character-string constants.  The subscripts
#              are parameter names like 'indentation', 'pagewidth', and 'markasliving'
#    $draw - What amounts to an array of string constants, each consisting of one printed character,
#            though many of them are entities. The subscripts are character names such as
#              "vbarOn", 'spousebarOn', 'cornerMale', 'cornerFemale'.
#            Note that some of these strings may be  applicable only to the Text+ Descendancy chart,
#            and others may be applicable only to the Text+ Ancestor chart.
#    $drawbar - An array of one-printed-character string constants that draw the vertical lines
#               and the characters that interrupt the verical lines. Specifically:
#               0 => space, 1=> vertical bar.
#               Characters 2 and 3 are different for Pedigree and Ancestor reports
function initializeParameters($whichreport) {
	global $params, $draw, $drawvbar, $vbars; #The arrays we are creating
	global $tngprint; #The pretty-print flag
	global $pedigree, $sitever, $rrconfig; #TNG system variables
	#Querystring parameters common to both reports
	global $generations, $fontsize, $indentation, $pagewidth, $widthplus, $height, $gensallowed;
	global $dpoptions;

if (empty($rrconfig['Text+MainWidth'])) $rrconfig['Text+MainWidth'] = 'landscape';
if (empty($rrconfig['Text+PrintWidth'])) $rrconfig['Text+PrintWidth'] = 'portrait';
if (!isset($rrconfig['Text+UserDatePlace'])) $rrconfig['Text+UserDatePlace'] = '1';
if (!isset($rrconfig['Text+Date'])) $rrconfig['Text+Date'] = 'full';
if (!isset($rrconfig['Text+3Lines'])) $rrconfig['Text+3Lines'] = '0';
if (!isset($rrconfig['Text+Place'])) $rrconfig['Text+Place'] = '1';
if (!isset($rrconfig['Text+SamePlace'])) $rrconfig['Text+SamePlace'] = '1';
if (!isset($rrconfig['Text+Age'])) $rrconfig['Text+Age'] = '';
if (!isset($rrconfig['Text+ShowMarriage'])) $rrconfig['Text+ShowMarriage'] = 1;
if (!isset($rrconfig['Text+AncestorMarriage'])) $rrconfig['Text+AncestorMarriage'] = 'parents';
if (!isset($rrconfig['Text+AncFooter'])) $rrconfig['Text+AncFooter'] = '1';
if (!isset($rrconfig['Text+AncestralLines'])) $rrconfig['Text+AncestralLines'] = '1';
if (empty($rrconfig['Text+DescFooterAppears'])) $rrconfig['Text+DescFooterAppears'] = 'prune';
if (!isset($rrconfig['Text+DescFooterReopens'])) $rrconfig['Text+DescFooterReopens'] = '1';
if (!isset($rrconfig['Text+AncIconInAnc'] )) $rrconfig['Text+AncIconInAnc'] = '0';
if (!isset($rrconfig['Text+DescIconInAnc'] )) $rrconfig['Text+DescIconInAnc'] = '0';
if (!isset($rrconfig['Text+AncIconInDesc'])) $rrconfig['Text+AncIconInDesc'] = '1';
if (!isset($rrconfig['Text+DescIconInDesc'])) $rrconfig['Text+DescIconInDesc'] = '1';
if (!isset($rrconfig['Text+LeafLinkOmit'])) $rrconfig['Text+LeafLinkOmit'] = '1';
#Not implemented as an option
if (!isset($rrconfig['Text+AncestorTracking'])) $rrconfig['Text+AncestorTracking'] = 1;
#Override options with end-user datePlace options or DatePlace Cookie
$params['DatePlaceCookie'] = 0;
if (!empty($dpoptions)) {
	extractDatePlaceOptions($dpoptions);
} else {
	$cookieName = "tng_{$params['phpBaseFilename']}_DatePlace";
	if (isset($_COOKIE[$cookieName])) {
		extractDatePlaceOptions($_COOKIE[$cookieName]);
		$params['DatePlaceCookie'] = 1;
	}
}

	for ($i=0; $i<30; $i++) $vbars[$i] = '';

	#If this is the Format for Printing page, then get the cookie that matches the chart type,
	#and extract the values for the four printer control parameters.
	$params['cookie'] = 0;
	$cookieName = "tng_{$params['phpBaseFilename']}";
	if ($tngprint || isset($_COOKIE[$cookieName])) {
		$cookieValue = $_COOKIE[$cookieName];
		if (strpos($cookieValue, '|') !== false) {
			$params['cookie'] = 1; #Tells us we used a cookie.
			$cookieValues = explode('|', $cookieValue);
			if (!$indentation) $indentation = $cookieValues[0];
			if (!$fontsize) $fontsize = $cookieValues[1];
			if (!$pagewidth) $pagewidth = $cookieValues[2];
			if ($widthplus=='') $widthplus = $cookieValues[3];
			if (!$height) $height = $cookieValues[4];
		}
	}
	#Number of generations to do, based on the querystring parameter $generations
	if ($generations===0 || $generations==='0') {
		$params['genstodo'] = 99; #0 means to go as far as we can.
	} elseif (!$generations) {
		#Use the appropriate system parameter, if it is defined.
		if ($whichreport == 'descend')
			$params['genstodo'] = $pedigree['initdescgens'] ? $pedigree['initdescgens'] : 4;
		else
			$params['genstodo'] = $pedigree['initpedgens'] ? $pedigree['initpedgens'] : 4;
	}
	else
		$params['genstodo']  = intval( $generations );

	#Maximum number of generations that can be selected in the generations selection box,
	#based on the querystring parameter $gensallowed.
	if ($gensallowed)
		$params['gensallowed'] = $gensallowed;
	elseif ($whichreport == 'descend')
		$params['gensallowed'] = 15;
	else
		$params['gensallowed'] = 18;

	# Finalize the printer control that can be driven by querystring params or cookie values,
	# or, in the case of page width, by TNG system variables.
	if (!$fontsize) $fontsize = 9; #points
	$params['fontsize'] = $fontsize;
	if (!$indentation) $indentation = 6; #spaces
	$params['indentation'] = $indentation;
	if (!$widthplus) $widthplus=0; #spaces
	$params['widthplus'] = $widthplus;
	if (!$height) $height = 100;
	$params['height'] = $height;

	#If we didn't get a page width from the Format for Printing cookie or a querystring parameter,
	# - Get the page type, possibly from one of the two TNG system variables that were added by this mod.
	# - Then compute the page width from the page type.
	if (!$pagewidth) {
		if ($sitever=='mobile') $startType= 'mobile';
		elseif ($tngprint)
			$startType = $rrconfig['Text+PrintWidth'];
		else $startType = $rrconfig['Text+MainWidth'];
		if (!$startType) $startType = 'landscape';
		$startType = strtolower($startType);

		if ($startType == 'mobile') $pagewidth = 4;
		elseif ($startType == 'landscape') $pagewidth = 10;
		elseif ($startType == 'legal') $pagewidth = 13;
		else $pagewidth = 7.5;
	}
	$params['pagewidth'] = $pagewidth;

	# Parameters derived from the font size
	$params['fixedCPI']  = 120./$fontsize * 1.25;
	$params['proportionalCPI'] = 120./$fontsize *1.2;

	# One-printed-character "constants"
	$draw = array('vbarOn' => '&#9474;',
		'vbarOff' => '&nbsp;',
		'spousebarOn' => ':',
		'spousebarOff' => '&nbsp;',
		'cornerMale' => '&#9484',
		'cornerFemale' => '&#9492',
		'vbarChild' => '&#9500',
		'vbarLastChild' => '&#9492',
		'hbar' => '&#9472',
		'hspaces' => '&nbsp;',
		'wrap' => '&nbsp;');

	# The first two characters that draw (or not) vertical bars.
	$drawvbar = array($draw['vbarOff'], $draw['vbarOn']);

	#Two of the $draw characters and two of the $drawvbar characters are different for the
	#pedigree & descendant reports.
	if ($whichreport == 'descend') {
		# Descendant chart $drawvbar characters:
		# 2=> the character for the corner that precedes the last child in a family, and
		# 3=> The character that consists of a vbar and a horizontal mark on the right to
		#      connect to horizontal bars.
		$drawvbar[2] = $draw['vbarLastChild'];
		$drawvbar[3] = $draw['vbarChild'];
	} else {
		# The hbar and hspaces $hbar character are doubled for the pedigree chart.
		$draw['hbar'] .= $draw['hbar'];
		$draw['hspaces'] .= $draw['hspaces'];
		# Pedigree chart $drawvbar characters:
		# 2=> The lower corner for the  the mother, and
		# 3=> The upper corner for the father.
		$drawvbar[2] = $draw['cornerFemale'];
		$drawvbar[3] = $draw['cornerMale'];
	}
} #end function intializeParameters

##########################################################
#Load the option settings with values from the consolidated end-user date/place options,
#which could have come from from a form parameter or a cookie.
function extractDatePlaceOptions($dpoptions) {
	global $rrconfig;
	$dpoption = substr($dpoptions,0,1);
	if ($dpoption=='f') $rrconfig['Text+Date'] = 'full';
	elseif ($dpoption=='y') $rrconfig['Text+Date'] = 'year';
	else $rrconfig['Text+Date'] = '';
	$rrconfig['Text+Place'] = substr($dpoptions,1,1);
	$rrconfig['Text+3Lines'] = substr($dpoptions,2,1);
	$rrconfig['Text+SamePlace'] = substr($dpoptions,3,1);
	$rrconfig['Text+Age'] = substr($dpoptions,4,1);
	$rrconfig['Text+ShowMarriage'] = substr($dpoptions,5,1);
	if (strlen($dpoptions) == 7) {
		$dpoption = substr($dpoptions,6,1);
		if ($dpoption=='p') $rrconfig['Text+AncestorMarriage'] = 'parents';
		elseif ($dpoption=='c') $rrconfig['Text+AncestorMarriage'] = 'child';
		else $rrconfig['Text+AncestorMarriage'] = '';
	}
} #end function extractDatePlaceOptions

####################################################################
#Display generation numbers and (optionally) sibling numbers just before the name
function PrintGenerationNumbers2($generations, $siblings='', $showsiblings=false) {
	//Print $generations right justified in a field of 2 spaces.
	//If $siblings is non-zerio print a period and $j left justified in two spaces.
	global $params;
	$myoutput = '';
	if ($generations == 0) $myoutput = '1';
	if ($generations != '') $myoutput .= $generations;
	if ($params['siblingnums'] && $siblings)
		$myoutput .= ".$siblings";
	return $myoutput;
} #end function PrintGen2

####################################################################
/** Function to display generation numbers padded with spaces. Primarily intended to be
* used at the beginning of a line. (Not used at all right now.)
* $i is the generation number; $j is the count of siblings within a generation
*    $i is right justified in a field of 2 spaces.
* If $j is 0,add 3 blank spaces. Otherwise, display a period and $j left justified in two spaces.
*/
function PrintGenerationNumbers($i, $j) {
	global $params;
	$myoutput = '';
	if ($i) $myoutput .= sprintf("%2d",$i);
	else $myoutput .= '  ';
	if (!$params['siblingnums']) return str_replace(' ', '&nbsp', $myoutput);
	if ($j) $myoutput .= ".".sprintf("%2d",$j);
	else $myoutput .= '   ';
	return str_replace(' ', '&nbsp', $myoutput);
} #end function PrintGenerationNumbers

#####################################
#Generate a legend for up to three chart link icons
# $charttype is 'Anc' or 'Desc' since those are the trailing portion of the applicable mod options
function generateIconLinksLegend($charttype) {
	global $tngprint, $rrconfig, $text;
	if ($tngprint) return '';
	$html = '';
	if ($rrconfig["Text+AncIconIn$charttype"])
		$html .= "<input type='checkbox' name='achart' id='achart' value='1'checked
			onclick=\"$('.achart').toggle();\"><img src='img/rrtextplus_achart.gif'>={$text['tp-printanc']}&emsp;\n";
	$html .= "<img src='img/rrtextplus_locate.gif'>={$text['tp-locateperson']}&emsp;\n";
	if ($rrconfig["Text+DescIconIn$charttype"])
		$html .= "<input type='checkbox' name='dchart' id='dchart' value='1'checked
			onclick=\"$('.dchart').toggle();\"><img src='img/rrtextplus_dchart.gif'>={$text['tp-printdesc']}\n";
	if ($html) $html = "<div>$html</div>\n";
	return $html;
} #end function generateIconLinksLegend

#####################################
#Generate up to 3 icons that link to charts for this person/
#In earlier versions of TextPlus charts, this did display up to 3 icons,
#two of them have been eliminated, leaving just the one "locate person" icon.
function generateIconLinks($row, $charttype, $trail='') {
	global $tngprint, $rrconfig, $text, $tree;
	$html = '';
	if ($tngprint) return '';
	if ($rrconfig["Text+AncIconIn$charttype"])
		$html .= "<a href='rrtextplus_ancestor.php?personID={$row['personID']}&amp;tree=$tree'
		title='{$text['tp-printanc']}' class='achart'>&thinsp;<img src='img/rrtextplus_achart.gif'></a>";
	if ($trail)
		$html .= "&thinsp;<img src='img/rrtextplus_locate.gif' title='{$text['tp-locateperson']}'
		class='rrtpTracker rrtpIcon' trail='$trail' width='9' height='9' alt='' border='0'>&thinsp;";
	if ($rrconfig["Text+DescIconIn$charttype"])
		$html .= "&thinsp;<a href='rrtextplus_descendant.php?personID={$row['personID']}&amp;tree=$tree'
		title='{$text['tp-printdesc']}' class='dchart'><img src='img/rrtextplus_dchart.gif'
		width='9' height='9' border='0'></a>&thinsp;";
	return $html;
} #end function generateIconLinks

######################################################################
# The document.ready function will copy the real footer's contents to the floating footer
# In TextPlus descendant reports, the footer contents change as tree branches are
# hidden or restored.
function buildFloatingFooter() {
	global $text;
	return "<div id='rrtpSummaryWrapper'>
	<div id='rrtpFloatingSummary' class='tpnoprint'>
	<div id='rrtpSummaryBanner' class='rrtpBoxBanner'>
		<div class='rrtpBoxButton'><img src='img/tng_close.gif' class='rrtpCloseX'></div>
		<div class='rrtpBoxButton'><img src='img/rrtextplus_top.png' class='rrtpLinkToTop'></div>
		<div>{$text['tp-chartsummary']}</div>
	</div>
	<div id='rrtpSummaryText' class='rrtpBoxText'></div>
	</div></div>\n"; #end rrtpFloatingSummary and rrtpSummaryWrapper
} #End buildFloatingFooter

######################################################################
# Insert Chart Types help link if the Chart Types Help mod has been installed
# Otherwise, just insert a label
function generateChartTypeLabel($charttype) {
	global $text, $modspath, $mylanguage;
	$html = '';
	if (!empty($text['rrcth-modfolder'])) {
		echo "<link href='css/rrcharttypehelp.css' rel='stylesheet' type='text/css'>
			<script src='js/rrcharttypehelp.js' type='text/javascript'></script>\n";
		#The Chart Type Help mod subfolder name is in a language string.
		$modfolder = $text['rrcth-modfolder'];
		echo "<div id='rrcthhelp' style='display:none;'>\n";
			if (file_exists("$modspath/$modfolder/{$mylanguage}_{$charttype}ext.php"))
				include("$modspath/$modfolder/{$mylanguage}_{$charttype}text.php");
			else include("$modspath/$modfolder/languages/English_{$charttype}text.php");
		echo "</div>\n";
		$html .= "<a href='#' onclick=\"return rrcthShowPopupHint(545,620);\"
			class='lightlink'>{$text['rrcth-charttypes']}:<img src='img/info_2.png'></a> &ensp; ";
	} else $html .= "{$text['tp-charttypes']}: &ensp;";
	return $html;
} #End function generateChartTypeLabel

#######################################################################
# Generate the date/place options form that will be displayed in a Litbox
function generateDatePlaceOptions() {
	global $text, $dateplace, $rrconfig, $params;
$html = "<div id='rrdpDrop' class='slidedown'>
<div class='rrtpBoxButton'><img class='rrtpCloseX' src='img/tng_close.gif' 
	onclick='closeDatePlaceSubform();'></div>
<div style='text-align:center;line-height:20px'><b>{$text['tp-dateplacetitle']}</b></div>
<table class='rrdpFormLayout'>\n";
	### Date format
	$checked1 = $rrconfig['Text+Date']=='full' ? ' checked' : '';
	$checked2 = $rrconfig['Text+Date']=='year' ? ' checked' : '';
	$checked3 = $rrconfig['Text+Date']=='' ? ' checked' : '';
	$html .= "<tr><td>{$text['tp-dateformat']}</td>
		<td><label class='nw'><input type='radio' name='dpDate' value='f'$checked1>{$text['tp-fulldate']}</label>
		<br><label class='nw'><input type='radio' name='dpDate' value='y'$checked2>{$text['tp-yearonly']}</label>";
	$html .= "</td></tr>\n";

	### Show Place
	$checked = $rrconfig['Text+Place'] ? ' checked' : '';
	$html .= "<tr><td>{$text['tp-showplace']}</td>
		<td><input type='checkbox' name='dpPlace' value='1'$checked onclick='dpSameDateFormat();'>
	</td></tr>\n";

	### When birth and death dates are the same
	if (false) {
	$checked1 = !$rrconfig['Text+SamePlace'] ? ' checked' : '';
	$checked2 = $rrconfig['Text+SamePlace'] ? ' checked' : '';
	$html .= "<tr id='dpSamePlace'><td>{$text['tp-sameplace']}
		<td><div style='margin-left:2em; text-indent:-2em;'>
			<label><span class='nw'><input type='radio' name='dpSamePlace' value='0'$checked1>
			Conventional: <b>5 Oct 1830 Paris, France -</b></span> <b class='nw'>Aug 1887 Paris, France</b></label>
			</div><div style='margin-left:2em; text-indent:-2em;'>
			<label><span class='nw'><input type='radio' name='dpSamePlace' value='1'$checked2>
			Compact: <b>Paris France;</b></span><b class='nw'>5 Oct 1830 - Aug 1887</b></label>
			</div>
	</td></tr>\n";
	}

	#### Include Age
	$checked = $rrconfig['Text+Age'] ? ' checked' : '';
	$html .= "<tr><td>{$text['tp-showage']}</td>
		<td><input type='checkbox' name='dpAge' value='1'$checked>
	</td></tr>\n";

	#### Show marriage data
	$checked = $rrconfig['Text+ShowMarriage'] ? ' checked' : '';
	$html .= "<tr><td>{$text['tp-showmarriage']}</td>
		<td><input type='checkbox' name='dpShowMarriage' value='1'$checked
			onclick='return dpSetAncestorMarriage()'>
	</td></tr>\n";

	if (false && $params['charttype'] == 'ancestor') {
		###Location of marriage data in ancestry chart
		$checked1 = $rrconfig['Text+AncestorMarriage']=='parents' ? ' checked' : '';
		$checked2 = $rrconfig['Text+AncestorMarriage']=='child' ? ' checked' : '';
		$checked3 = $rrconfig['Text+AncestorMarriage']=='' ? ' checked' : '';
		$html .= "<tr id='dpAncestorMarriage'><td>{$text['tp-marriageprompt']}</td>
			<td><label class='nw'><input type='radio' name='dpAncestorMarriage'
			value='p'$checked1>{$text['tp-bothparents']}</label>
		&ensp; <label class='nw'><input type='radio' name='dpAncestorMarriage'
			value='c'$checked2>{$text['tp-theirchild']}</label>
		&ensp; <label class='nw'><input type='radio' name='dpAncestorMarriage'
			value='n'$checked3>{$text['tp-neither']}</label>
		</td></tr>\n";
	}
	### 3-line display
	$checked = $rrconfig['Text+3Lines'] ? ' checked' : '';
	$html .= "<tr><td colspan='2'>{$text['tp-threelines']}: 
		<input type='checkbox' name='dp3Lines' value=''$checked onclick='dpSameDateFormat();'>
	</td></tr>\n";

	#Cookie & Submit buttons
	$html .= "<tr><td colspan='2'style='text-align:left;'>
		<input type='button' id='savedateplacecookie' value='{$text['tp-savesettings']}'
		class='small' onclick=\"saveDatePlaceCookie('{$params['phpBaseFilename']}');\"/>\n";
	#The Clear Cookie button is hidden if there is no cookie to clear.
	$cookiestyle = ($params['DatePlaceCookie']) ? '' : " style='display:none;'" ;
	$html .= "&nbsp; <input type='button' id='cleardateplacecookie' class='small'$cookiestyle
		value='{$text['tp-clearcookie']}'
		onclick=\"clearDatePlaceCookie('{$params['phpBaseFilename']}');\"/>\n";
	#Submit button
	$html .= "<div style='float:right;'>
		<input type='submit' name='button' onclick='submitDatePlaceSubform();'
			value='{$text['tp-redisplay']}'>
			</div>
		</td></table>
		</div>\n";
	return $html;
} #End function generateDatePlaceOptions
?>
